﻿using System;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace FanduinaV3CS
{

    public partial class FanduinoInterface : Form //ファンコンとデータを送受信するクラス
    {
        #region "ClassDef"
        //プロパティ
        public String   PortName                    //接続ポート名（COM*，HID*）
        { 
            get{ return portName; }
            set 
            {
                portName = value;
                comboBoxPort.Text = value;
           }
        }
        public Int32    FicNum                      //登録Fic番号
        {
            get{return Convert.ToInt32(FicNumber.Text);}
            set{FicNumber.Text = value.ToString();}
        }
        public Int32    FanCount                    //接続可能ファン台数
        { get; set; }
        public Int32    TempCount                   //接続可能温度計台数
        { get; set; }
        public Int32    PwmCount                    //ファンPWM数
        { get; set; }
        public string   Firm                        //ファーム名称
        { get; set; }
        public bool     IsCSFAvalable               //低温時ファン停止機能(CSF)があるかどうか
        { get; set; }
        public bool     IsManualAvalable            //手動モードがあるかどうか
        { get; set; }
        public bool     IsTestModeOn                //テストモードかどうか
        { get; set; }
        public FanDataStruct[]    FanData   = new FanDataStruct[FicFanMax];     //ファンの名称、回転数、デューティ
        public TempDataStruct[]   TempData  = new TempDataStruct[FicTempMax];   //温度計の名称、温度
        public FanParamStruct[]   FanParam  = new FanParamStruct[FicFanMax];    //ファン制御パラメータ
        public TempParamStruct[]  TempParam = new TempParamStruct[FicTempMax];  //温度計パラメータ
        public FicConfigDialog    FicDialog;            						 //Ficの設定ダイアログ
        //コンストラクタ
        public FanduinoInterface()     
        {
            InitializeComponent();
            for (int i = 0; i < FicFanMax; i++)
            {
                FanParam[i] = new FanParamStruct(FicNum, i);
                FanData[i]  = new FanDataStruct(FicNum, i);
            }
            for (int i = 0; i < FicTempMax; i++)
            {
                TempParam[i] = new TempParamStruct(FicNum, i);
                TempData[i]  = new TempDataStruct(FicNum, i);
            }
            IsTestModeOn = false;
            Load += FanduinoInterface_Load;
        }
        //メソッド
        public void PortConnect()               //ポートに接続する
        {
            try
            {
                PortClose();
                PortEnum();
                PortOpen(portName);
                FanduinaSettings.FicProp[FicNum].PortName = portName;
            }
            catch
            {
            }
        }
        public void PortOpen(string port)       //ポートを開く
        {
            try
            {
                if (port == "off")
                {
                    //Port off
                    portType = PORTTYPE_OFF;
                }
                else if (port.Substring(0, 3) == "HID")
                {
                    //USB-HID Port
                    HidPortOpen(port);
                    portType = PORTTYPE_HID;
                }
                else if (port.Substring(0, 3) == "COM")
                {
                    //Serial COM Port
					string portTTY = "/dev/ttyUSB" + port.Substring(3, 1);	//V3CSと相違
                    comPortOpen(portTTY);	//V3CSと相違
                    portType = PORTTYPE_COM;
                }
            }
            catch
            {
            }
        }
        public void PortClose()                 //接続中のポートを閉じる
        {
            try
            {
                if (comPort.IsOpen == true)
                {
                    comPortClose();
                }
                if (hidPort.IsOpen == true)
                {
                    HidPortClose();
                }
                ClearInfo();
            }
            catch
            {
            }
        }
        public void PortEnum()                  //ポート候補をcomboBoxに列挙する
        {
            comboBoxPort.Items.Clear();
            comboBoxPort.Items.Add("off");
            try
            {
                HidPortEnum();
                if (hidEnum.NumberOfDevs > 0)
                {
                    for (int i = 0; i <= hidEnum.NumberOfDevs - 1; i++)
                    {
                        comboBoxPort.Items.Add(hidEnum.HidName[i]);
                    }
                }

                foreach (string sp in System.IO.Ports.SerialPort.GetPortNames())
                {
					//V3CSと相違
					try
					{
						if (sp.Substring(0,11) == "/dev/ttyUSB")
						{
							string comName = "COM" + sp.Substring(11, 1);
                    		comboBoxPort.Items.Add(comName);
						}
					}
					catch
					{
					}
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }
        //フィールド変数
//        private MainForm        mainForm;	//V3CSと相違
        private FicFanInfo[]    FanInfo   = new FicFanInfo[FicFanMax];
        private FicTempInfo[]   TempInfo  = new FicTempInfo[FicTempMax];
        private Int32           portType;
        private string         portName;
		private string 		 exeDir;	//このプログラムのexeファイルが格納されているディレクトリ	//V3CSと相違
        //定数
        private const Int32     FicFanMax  = 6;
        private const Int32     FicTempMax = 4;
        private const Int32     PORTTYPE_OFF = -1;
        private const Int32     PORTTYPE_COM = 0;
        private const Int32     PORTTYPE_HID = 1;
        //ロード時初期化
        private void FanduinoInterface_Load(object sender, EventArgs e) //Ficフォームを開き、ファンと温度の表示エリアを設定する
        {
			exeDir = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location).ToString();	//V3CSと相違
//			MessageBox.Show("Fic: " + FicNum.ToString() + " : " + exeDir );
            //現在値表示エリアの作成
            for (int i = 0; i < FicFanMax; i++)
            {
                FanInfo[i]  = new FicFanInfo(FicNum, i);
                groupBoxFan.Controls.Add(FanInfo[i]);
                FanInfo[i].Location = new Point(5, groupBoxFan.Size.Height);
                groupBoxFan.Size = (System.Drawing.Size)new Point(groupBoxFan.Size.Width,
                                     groupBoxFan.Size.Height + FanInfo[i].Size.Height + 2);
            }
            groupBoxFan.Size = (System.Drawing.Size)new Point(groupBoxFan.Size.Width,
                                                              groupBoxFan.Size.Height + 4);
            for (int i = 0; i < FicTempMax; i++)
            {
                TempInfo[i]  = new FicTempInfo(FicNum, i);
                groupBoxTemp.Controls.Add(TempInfo[i]);
                TempInfo[i].Location = new Point(5, groupBoxTemp.Size.Height);
                groupBoxTemp.Size = (System.Drawing.Size)new Point(groupBoxTemp.Size.Width,
                                     groupBoxTemp.Size.Height + TempInfo[i].Size.Height + 2);
            }
            groupBoxTemp.Size = (System.Drawing.Size)new Point(groupBoxTemp.Size.Width,
                                                               groupBoxTemp.Size.Height + 4);
            //ボタンの設定
            dispSlow = false;
            buttonSlowDown.Text = Properties.Resources.SpeedSlow;
            dispPause = false;
            buttonPause.Text = Properties.Resources.SerialDataPause;
			
//            mainForm = (MainForm)this.Owner;	//V3CSと相違			
            FicRev = new FanduinoRevision();
//            ShowInTaskbar = false;	//V3CSと相違
            this.timerLog.Interval = FanduinaSettings.FicProp[FicNum].LogInterval * 1000;  //ログインターバルの設定復元
        }
        #endregion

        #region "FormEvents" //フォームの処理
        private void buttonClose_Click(object sender, EventArgs e)          //Ficフォームを閉じる
        {
            this.Hide();
         }
        private void comboBoxPort_Enter(object sender, EventArgs e)         //接続ポートの一覧をcomboBoxに表示
        {
            PortEnum();
        }
        private void comboBoxPort_TextChanged(object sender, EventArgs e)   //選択されているポート名が変更された時
        {
            portName = comboBoxPort.Text;
        }
        private void buttonConnect_Click(object sender, EventArgs e)        //接続ボタンが押された時
        {
            PortConnect();
        }
        private void buttonConfig_Click(object sender, EventArgs e)         //設定用のFicDiaLogを呼び出す
        {
			if ( FicDialog == null || FicDialog.IsDisposed )		//FicDialogは１つだけしか開けないようにする
			{
            	FicDialog = new FicConfigDialog();
            	FicDialog.Owner = this;
            	FicDialog.FicNum = this.FicNum;
            	FicDialog.Show();
            	FicDialog.FormClosed += ficDialog_Close;
            	FicDialog.SendEvent  += ficDialog_Send;
            	//FicDialog.ExThermoEnumEvent += ExThermoEnumEventAction;
			}
        }
        private void ficDialog_Close(object sender, EventArgs e)            //FicDialogの終了処理
        {
            FicDialog.Dispose();
            System.GC.Collect();
        }
        private void ClearInfo()    //FanInfoおよびTempInfoをクリアする
        {
            for (int i = 0; i < FicFanMax; i++)
            {
                FanData[i].FanName  = " ";
                FanData[i].FanTach  = 0;
                FanData[i].FanDuty  = 0.0;
                FanInfo[i].FanData  = FanData[i];
            }
           for (int i = 0; i < FicTempMax; i++)
            {
                TempData[i].TempName = " ";
                TempInfo[i].TempData = TempData[i];
            }
        }
        #endregion

        #region "SerialData" //データ送受信時の処理
        private String serialReceived;  //受信データ
        private String serialSend;      //送信データ
        private FanduinoRevision FicRev = new FanduinoRevision(); 			//ファームバージョンとその機能
        private bool dispSlow  = false; //低速表示
        private bool dispPause = false; //ポーズ
        private bool dispSend  = false; //送信画面を表示

        private void ficDialog_Send(object sender, SendEventArgs e)         //イベントによるデータ送信処理
        {
            int bufLen = 0;
            for (int i = 0; i < e.LineNum; i++)
            {
                serialSend = e.SendData[i];
                if (portType == PORTTYPE_COM)
                {
                    if (bufLen > 200)   //送信データの累積バイト数が大きくなったら、送信を4秒間中断
                    {
                        System.Threading.Thread.Sleep(4000);
                        bufLen = 0;
                    }
                    bufLen += serialSend.Length;
                    comPort_DataSend();
                }
                else if (portType == PORTTYPE_HID)
                {
                    HidPortSend();
                }
                if (dispSend) ListDataUpdate(serialSend);
            }
            e.Success = true;
        }
        delegate void SetTextCallback(string text);                         //デリゲート宣言
        private void SetText(string text)                                   //SerialDataにtextを代入するデリゲート関数
        {
            if (!dispSend) ListDataUpdate(text);   //listSerialDataLinesを更新
            ReceivedData(text);     //SidePanelに表示
        }
        private void ReceivedData(string strDataReceived)                   //受信データをSidePanelに表示
        {
            string reportItem = null;
            string reportItemSetting = null;
            string[] stArray = new string[13];

            FanDataStruct  fanData;
            TempDataStruct tempData;

            //受信データの表示
            try
            {
                stArray = strDataReceived.Replace(",", "\n").Split('\n');
            }
            catch
            {
            }

            if (stArray.Length >= 1)
            {
                reportItem = stArray[0].Trim();

                //ファンデータの場合
                if (reportItem == "Fan")
                {
                    try
                    {
                        Int32  fanNum     = Convert.ToInt32(stArray[1]);
                        string fanName    = stArray[2].Trim();
                        string fanTachStr = stArray[3].Trim(); //(rpm)
                        string fanDutyStr = stArray[4].Trim(); //(%)

                        fanData.FicNum  = FicNum;
                        fanData.FanNum  = fanNum - 1;
                        fanData.FanName = fanName;
                        fanData.FanTach = Convert.ToInt32(fanTachStr);
                        fanData.FanDuty = Convert.ToDouble(fanDutyStr);
                        FanInfo[fanNum - 1].FanData = fanData;
                        FanData[fanNum - 1] = fanData;

                        FicDialog.FanTach = fanData.FanTach;
                        FicDialog.FanTachNum = fanData.FanNum;
                     }
                    catch
                    {
                    }

                //温度計データの場合
                }
                else if (reportItem == "Thm")
                {
                    try
                    {
                        Int32 tempNum       = Convert.ToInt32(stArray[1]);
                        string tempName     = stArray[2].Trim();
                        string tempValueStr = stArray[3].Trim();   //（℃）

                        tempData.FicNum    = FicNum;
                        tempData.TempNum   = tempNum - 1;
                        tempData.TempName  = tempName;
                        tempData.TempValue = Convert.ToDouble(tempValueStr);
                        TempInfo[tempNum - 1].TempData = tempData;
                        TempData[tempNum - 1] = tempData;
                    }
                    catch
                    {
                    }

                //ファンコンパラメータの場合
                }
                else if (reportItem == "Prm")
                {
                    Int32 maxPWM  = FicRev.MaxPWM;
                    Int32 maxFan  = FicRev.MaxFan;
                    Int32 maxTemp = FicRev.MaxTemp;
                    Int32 dmyFan = maxFan - maxPWM;
                    try
                    {
                        reportItemSetting = stArray[1].Trim();
                        switch(reportItemSetting)
                        {
                            case "Tmin": //ファン制御最低温度（℃）                  
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Tmin = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Tmin = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "Tmax": //ファン制御最高温度（℃）
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Tmax = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Tmax = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "Tstop": //ファン停止温度（℃）
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Tstop = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Tstop = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "Dmin": //ファン制御最低温度でのDuty比（％）
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Dmin = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Dmin = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "Dmax": //ファン制御最高温度でのDuty比（％）
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Dmax = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Dmax = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "Dman": //ファン手動制御時のDuty比（％）
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].Dman = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].Dman = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "FanThm": //ファン制御を行う温度計番号
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].FanThm = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].FanThm = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "FanMan": //ファン自動／手動設定
                                for (int fan = 0; fan < maxPWM; fan++) { FanParam[fan].FanMan = Convert.ToInt32(stArray[fan + 2]); }
                                if (dmyFan > 0)
                                {
                                    for (int fan = maxPWM; fan <= maxFan; fan++) { FanParam[fan].FanMan = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                }
                                break;
                            case "FanNam": //ファン名称
                                for (int fan = 0; fan < maxFan; fan++)
                                {
                                    string fname = stArray[fan + 2].Trim();
                                    FanParam[fan].FanName  = fname.PadRight(6,' ');
                                }
                                break;
                            case "ThmNam": //温度計名称
                               for (int tmp = 0; tmp < maxTemp; tmp++)
                                {
                                    string tname = stArray[tmp + 2].Trim();
                                    TempParam[tmp].TempName  = tname.PadRight(6,' ');
                                }
                               SendExTemp();    //代用温度計データを送信
                                break;
                            case "Firm": //ファームバージョン
                                Firm = stArray[2].Trim();
                                labelFirm.Text = Firm;
                                FicRev.Firm = Firm;
                                FanCount  = FicRev.MaxFan;
                                TempCount = FicRev.MaxTemp;
                                PwmCount  = FicRev.MaxPWM;
                                FanduinaSettings.FicProp[FicNum].FanCount  = FanCount;
                                FanduinaSettings.FicProp[FicNum].TempCount = TempCount;
                                FanduinaSettings.FicProp[FicNum].PwmCount  = PwmCount;
                                IsCSFAvalable    = FicRev.IsCSFAvalable;
                                IsManualAvalable = FicRev.IsManualAvalable;
                                break;
                            default:
                                break;
                        }
                    }
                    catch
                    {
                    }
                }
                else if (reportItem == "TestOn") //テストモードOnの場合
                {
                    IsTestModeOn = true;
                }
                else if (reportItem == "TestOff") //テストモードOffの場合
                {
                    IsTestModeOn = false;
                }
            }
        }
        private void ListDataUpdate(string text)                            //送受信データをlistSerialDataLinesに表示
        {
            if (!dispPause)    //送受信データ表示を更新
            {
                listSerialDataLines.BeginUpdate();
                if (listSerialDataLines.Items.Count > 19)
                {
                    listSerialDataLines.Items.RemoveAt(19);
                }
                listSerialDataLines.Items.Insert(0, text);
                listSerialDataLines.EndUpdate();
            }
            if (dispSlow) { Thread.Sleep(100); }   //低速表示のときはウェイトを入れる
        }
        private void buttonSlowDown_Click(object sender, EventArgs e)       //表示速度の切替
        {
            if (dispSlow)
            {
                buttonSlowDown.Text = Properties.Resources.SpeedSlow;
                dispSlow = false;    //通常速度に切替
            }
            else
            {
                buttonSlowDown.Text = Properties.Resources.SpeedNormal;
                dispSlow = true;   //低速表示に切替
            }
        }
        private void buttonPause_Click(object sender, EventArgs e)          //一時停止の切替
        {
            if (dispPause)
            {
                buttonPause.Text = Properties.Resources.SerialDataPause;
                dispPause = false;      //通常表示に切替
                listSerialDataLines.BeginUpdate();
                if (listSerialDataLines.Items.Count > 19)
                {
                    listSerialDataLines.Items.RemoveAt(19);
                }
                listSerialDataLines.Items.Insert(0,
                    "================================================");
                listSerialDataLines.EndUpdate();
            }
            else
            {
                buttonPause.Text = Properties.Resources.SerialDataShow;
                dispPause = true;       //ポーズに切替
            }
        }
        private void receiveData_CheckedChanged(object sender, EventArgs e) //リスト表示の送受信を切替
        {
            if (receiveData.Checked)
            {
                dispSend = false;   //受信データ表示に切替
                listSerialDataLines.Items.Clear();
                listSerialDataLines.ForeColor = Color.Black;
            }
            else
            {
                dispSend = true;   //送信データ表示に切替
                listSerialDataLines.Items.Clear();
                listSerialDataLines.ForeColor = Color.Red;
            }
        }
        #endregion

        #region "SerialPort" //シリアルポート（COMポート）の処理
        private void comPortOpen(string port)   //シリアルポートを開く
        {
            try
            {
                comPort = new SafeSerialPort();	//V3CSと相違
                //プロパティセット
                comPort.PortName = port;
                comPort.BaudRate = 9600;
                comPort.Parity = 0;
                comPort.DataBits = 8;
                comPort.StopBits = StopBits.One;
                comPort.ReadBufferSize =  1024;
                comPort.WriteBufferSize = 1024;
				comPort.ReadTimeout = 8;
//                comPort.DiscardNull = false;	//V3CSと相違
//                comPort.RtsEnable = false;	//V3CSと相違
//                comPort.DtrEnable = true;	//V3CSと相違
               //既にオープンされていないか？
                if ((comPort.IsOpen == true))
                {
                    MessageBox.Show(comPort.PortName + Properties.Resources.ErrPortAlreadyOpened, 
                                    Properties.Resources.ErrorMessage, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                //ポートオープン
                comPort.Open();
				timerCom.Enabled = true;	//V3CSと相違
            }
            catch
            {
                    MessageBox.Show(comPort.PortName + Properties.Resources.ErrPortAlreadyOpened, 
                                    Properties.Resources.ErrorMessage, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void comPortClose()             //シリアルポートを閉じる
        {
            try
            {
                if ((comPort.IsOpen == true))
                {
					timerCom.Enabled = false;	//V3CSと相違
                    comPort.Close();
                    comPort.Dispose();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Properties.Resources.ErrorMessage,
                                   MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void comPort_DataSend()         //シリアルポートにデータ送信する
        {
            comPort.WriteLine(serialSend);
        }
        private void comPort_DataReceived(object sender, SerialDataReceivedEventArgs e) //シリアルポートに受信した時のイベント
        {
            try
            {
//                serialReceived = comPort.ReadLine().Trim();	//V3CSと相違
//                SetTextCallback dgSetText = new SetTextCallback(SetText);	//V3CSと相違
//                this.Invoke(dgSetText, serialReceived);	//V3CSと相違
            }
            catch
            {
            }
        }
        private void comPort_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
        {
        }
		private void ComPortReceive()	//V3CSと相違
		{
            try
            {
				if ( comPort.BytesToRead == 0 )
				{
					return;
				}
				byte comRead;
				serialReceived = "";
				do
				{
					comRead = (byte)255;
					comRead = (byte)comPort.ReadByte();
					switch (comRead)
					{
						case (byte)255		: return;
						case (byte)'\n'	: break;
						case (byte)'\r'	: break;
						default 			: serialReceived += (char)comRead; break;
					}
				} while( (serialReceived.Length < 61) & (comRead != (byte)'\n') );
				
				if ( !String.IsNullOrEmpty(serialReceived) )
				{
                	SetTextCallback dgSetText = new SetTextCallback(SetText);
                	this.Invoke(dgSetText, serialReceived.Trim());
				}
            }
            catch
            {
            }
		}
        private void timerCom_Tick(object sender, EventArgs e)  //timerComによるポーリング	//V3CSと相違
        {
            ComPortReceive();
        }
        #endregion

        #region "USB HID Port" //HIDポートの処理
        private const Int32     HID_ModelMax = 2;                   //使用できるHIDモデル数（Rev8.0, Rev6.0の2種類)
        private const Int32     HID_Control_Transfer = 0;           //コントロール転送タイプのID
        private const Int32     HID_Interrupt_Transfer = 1;         //インタラプト転送タイプのID
        private UInt16[]        HID_VendorID = {0x16c0,0x16d0};     //使用できるHIDモデルのベンダーID
        private UInt16[]        HID_ProductID = {0x27d9,0x4fe}; 	//使用できるHIDモデルのプロダクトID
        private Int32[]         HID_Type = {HID_Control_Transfer,	HID_Interrupt_Transfer};    //使用できるHIDモデルの転送タイプ
		private Int32[]		 	HID_Length = {32,64};				//使用できるHIDモデルのレコード長	//V3CSと相違
        private HidEnumClass    hidEnum;                            //接続されているHIDデバイスを列挙するインスタンス
        private HidStringClass  hidPort;                            //選択したHIDデバイスと文字列を通信するインスタンス
        private Int32           hidReportLength;                    //HIDデバイスとの通信文字列長
//        private byte[]          hidSendBuffer = new byte[65];       //送信用バッファ	//V3CSと相違
//        private byte[]          hidRecvBuffer = new byte[65];       //受信用バッファ	//V3CSと相違

        public void HidPortEnum()               //HIDポートを列挙する
        {
            bool result = false;
            int i = 0;
            hidEnum = new HidEnumClass();
            hidEnum.ModelMax = HID_ModelMax;
            for (i = 0; i < HID_ModelMax ; i++)
            {
                hidEnum.VendorID[i]  = HID_VendorID[i];
                hidEnum.ProductID[i] = HID_ProductID[i];
                hidEnum.HidType[i]   = HID_Type[i];
				hidEnum.RepLength[i] = HID_Length[i];
            }
            result = hidEnum.EnumHidDevs();
        }
        public void HidPortOpen(string port)    //HIDポートを開く
        {
            bool result = false;
            int HidNumber = 0;
            try
            {
                for (HidNumber = 0; HidNumber < hidEnum.NumberOfDevs; HidNumber++)
                {
                    if (port == hidEnum.HidName[HidNumber])
                    {
                        hidPort = new HidStringClass();
                        if (HidNumber >= 0 & HidNumber < 16)
                        {
                            hidPort.HidType   = HID_Type[hidEnum.HidModel[HidNumber]];
                            hidPort.ReportLength  = HID_Length[hidEnum.HidModel[HidNumber]];
                            hidPort.VendorID  = HID_VendorID[hidEnum.HidModel[HidNumber]];
                            hidPort.ProductID = HID_ProductID[hidEnum.HidModel[HidNumber]];
                            hidPort.VersionNO = hidEnum.VersionNO[HidNumber];
                            hidPort.SerialNO  = hidEnum.SerialNO[HidNumber];
							hidPort.Profile   = hidEnum.Profile[HidNumber];	//V3CSと相違
                            result = hidPort.Open();
                            if (result)
                            {
                                hidReportLength = hidPort.ReportLength;
                                timerHID.Enabled = true;	//V3CSと相違
                            }
                        }
                        break;
                    }
                }
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrPortOpen, Properties.Resources.ErrorMessage,
                                   MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void HidPortClose()             //HIDポートを閉じる
        {
            try
            {
                hidPort.Close();
                timerHID.Enabled = false;	//V3CSと相違
            }
            catch
            {
            }

        }
        private void HidPortReconnect()         //HIDポートを再接続する
        {
            HidPortClose();
            //ポートを閉じる
            System.Threading.Thread.Sleep(300);
            HidPortOpen(portName);
            //ポートを開く
        }
        private void HidPortSend()              //HIDポートへのデータ送信
        {
            bool result = hidPort.WriteString(serialSend);
        }
        private void HidPortReceive()           //HIDポートからのデータ受信
        {
             try
             {
                 serialReceived = hidPort.ReadString().Trim();
                 //HIDからデータを受信
                 if (!String.IsNullOrEmpty(serialReceived))
                 {
                     try
                     {
                         SetTextCallback dgSetText = new SetTextCallback(SetText);
                         this.Invoke(dgSetText, serialReceived);
                     }
                     catch
                     {
                         //MessageBox.Show("error");
                     }
                 }
                 else
                 {
                     //MessageBox.Show("null");
                 }
             }
             catch
             {
             }
        }
        private void timerHID_Tick(object sender, EventArgs e)  //timerHIDによるポーリング
        {
            HidPortReceive();
        }
        #endregion

        #region "Alarm" //ファンと温度の監視
        //イベント宣言
        public event EventHandler<AlarmFanEventArgs> AlarmFanEvent;    //ファン停止アラームイベント
        AlarmFanEventArgs alarmFanArgs = new AlarmFanEventArgs();
        public event EventHandler<AlarmTempEventArgs> AlarmTempEvent;  //温度超過継続アラームイベント
        AlarmTempEventArgs alarmTempArgs = new AlarmTempEventArgs();
        //ローカル変数
        private bool[]  fanStop  = new bool[FicFanMax];         //ファンが停止
        private Int32[] tempOverSec = new Int32[FicTempMax];    //温度が設定値を超えている秒数
        private bool[]  tempOver = new bool[FicTempMax];        //温度が設定値と時間を超過
        //ファンと温度の監視
        private void timerAlarm_Tick(object sender, EventArgs e)    //タイマーによる10秒ごとの監視
        {
            //ファンの監視
            CheckFanData();
            for (int i = 0; i < FanCount; i++)
            {
                if (fanStop[i])
                {
                    alarmFanArgs.FicNum = this.FicNum;
                    alarmFanArgs.FanNum = i;
                    alarmFanArgs.FanDuty = FanData[i].FanDuty;
                    alarmFanArgs.AlarmByBaloon = FanduinaSettings.FicProp[this.FicNum].AlarmByBalloon;
                    alarmFanArgs.AlarmBySound  = FanduinaSettings.FicProp[this.FicNum].AlarmBySound;
                    AlarmFanEvent(this, alarmFanArgs);
                }
            }
            //温度の監視
            CheckTempData();
            for (int i = 0; i < TempCount; i++)
            {
                if (tempOver[i])
                {
                    alarmTempArgs.FicNum = this.FicNum;
                    alarmTempArgs.TempNum = i;
                    alarmTempArgs.TempAlarm = FanduinaSettings.FicProp[this.FicNum].AlarmTempValue[i];
                    alarmTempArgs.TempValue = TempData[i].TempValue;
                    alarmTempArgs.AlarmByBaloon = FanduinaSettings.FicProp[this.FicNum].AlarmByBalloon;
                    alarmTempArgs.AlarmBySound  = FanduinaSettings.FicProp[this.FicNum].AlarmBySound;
                    AlarmTempEvent(this, alarmTempArgs);
                }
            }
        }
        private void CheckFanData()     //ファンが停止していたらfanSTopをtrueにする
        {
            for ( int i = 0; i < FanCount; i++ )
            {
                fanStop[i] = false;
                if (FanduinaSettings.FicProp[this.FicNum].AlarmFanEnable[i])
                {
                    if ((FanData[i].FanTach == 0) & (FanData[i].FanDuty > 0))
                    {
                        fanStop[i] = true;
                    }
                }
            }
        }
        private void CheckTempData()    //各温度が設定値を超えている秒数を累積し、上限秒数を超えたらtempOverをtrueにする
        {
            for ( int i = 0; i < TempCount; i++ )
            {
                tempOver[i] = false;
                if (FanduinaSettings.FicProp[this.FicNum].AlarmTempEnable[i])
                {
                    if (TempData[i].TempValue > (double)FanduinaSettings.FicProp[this.FicNum].AlarmTempValue[i])
                    {
                        tempOverSec[i] += (Int32)(timerAlarm.Interval / 1000);
                        if (tempOverSec[i] >= FanduinaSettings.FicProp[this.FicNum].AlarmTempTimer)
                        {
                            tempOverSec[i] = 0;
                            tempOver[i] = true;
                        }
                    }
                    else
                    {
                        tempOverSec[i] = 0;
                    }
                }
                else
                {
                    tempOverSec[i] = 0;
                }
            }
        }
        #endregion

        #region "Log"   //ファンと温度の記録
        private void   timerLog_Tick(object sender, EventArgs e)  //タイマーにより一定時間ごとに記録
        {
            if (FanduinaSettings.FicProp[FicNum].LogEnable)
            {
                bool result = LogWrite();
                if (!result)
                {
                    MessageBox.Show(Properties.Resources.ErrLogWrite, Properties.Resources.ErrorMessage,
                                       MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            this.timerLog.Interval = FanduinaSettings.FicProp[FicNum].LogInterval * 1000;
        }
        private bool   LogWrite()       //ログをファイルに追記する
        {
            string dirPath = Path.Combine(exeDir,"LogFiles") ;	//V3CSと相違
            if (!System.IO.File.Exists(dirPath))
            {
                System.IO.Directory.CreateDirectory(dirPath);
            }
            string startDateTime = DateTime.Now.ToString("yyyy-MM-dd");
            string currentLogFileName = "Log-" + startDateTime + "-" + FicNum.ToString().Trim() + "-" + FicRev.Firm.Trim() + ".txt";
            string filePath = dirPath + "/" + currentLogFileName;	//V3CSと相違
            string logHeader;
            string logLine;
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");

            if (!System.IO.File.Exists(filePath))
            {
                logHeader = LogHeader();
                if (logHeader.Length > 0)
                {
                    StreamWriter LogFileWrite = new StreamWriter(filePath, false, sjisEnc);
                    LogFileWrite.WriteLine(logHeader);
                    LogFileWrite.Close();
                }
            }
            logLine = LogLine();
            if (logLine.Length > 0)
            {
                StreamWriter LogFileWrite = new StreamWriter(filePath, true, sjisEnc);
                LogFileWrite.WriteLine(logLine);
                LogFileWrite.Close();
                return true;
            }
            else
            {
                return false;
            }
        }
        private string LogHeader()      //ログのヘッダー行を返すファンクション
        {
            string logHeader = DateTime.Now.ToString("yyyy-MM-dd");
            for (int i = 0; i < TempCount; i++)
            {
                if (TempParam[i].TempName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logHeader += "," + (TempParam[i].TempName).PadRight(6);
                }
            }
            for (int i = 0; i < FanCount; i++)
            {
                if (FanParam[i].FanName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logHeader += "," + (FanParam[i].FanName).PadRight(6);
                }
            }
            for (int i = 0; i < FanCount; i++)
            {
                if (FanParam[i].FanName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logHeader += "," + (FanParam[i].FanName).PadRight(6);
                }
            }
            return logHeader;
        }
        private string LogLine()        //ログの１行分を返すファンクション
        {
            string logLine = DateTime.Now.ToString("T") + " ";
            for (int i = 0; i < TempCount; i++)
            {
                if (TempParam[i].TempName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logLine += "," + (TempData[i].TempValue.ToString("0.0")).PadLeft(6);
                }
            }
            for (int i = 0; i < FanCount; i++)
            {
                if (FanParam[i].FanName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logLine += "," + (FanData[i].FanTach.ToString()).PadLeft(6);
                }
            }
            for (int i = 0; i < FanCount; i++)
            {
                if (FanParam[i].FanName.Length < 1)
                {
                    return "";
                }
                else
                {
                    logLine += "," + (FanData[i].FanDuty.ToString("0.0")).PadLeft(6);
                }
            }
            return logLine;

        }
        #endregion

        #region "ExThermo"  //代用温度計データの取得とファンコンへの送信
        private ExThermoEnumArgs exArgs;
        private SendEventArgs sdArgs;
        private void SendExTemp()   //代用温度を取得してファンコンに送信
        {
            double[] exThermoValue = new double[FicTempMax];
            bool isExThermoRequested = false;    //代用温度の送信が必要かどうか
            try
            {
                for (int i = 0; i < TempCount; i++)
                {
                    Int32 exThermoType = FanduinaSettings.FicProp[FicNum].ExThermoType[i];
                    string exThermoName = FanduinaSettings.FicProp[FicNum].ExThermoName[i];
                    exThermoValue[i] = 0.0;
                    if (exThermoType != ExThermo.OwnSensor)
                    {
                        exArgs = new ExThermoEnumArgs(exThermoType); //exArgsに代用温度計のタイプを格納
                        exArgs.FicNum = FicNum;
                        exArgs.TempNum = i;
                        bool rapid = false;
                        ExThermoClass.ExThermoEnumRequest(rapid, ref exArgs);    //代用温度をMainFormに問い合わせ
                        if (exArgs.Success)
                        {
                            for (int j = 0; j < exArgs.ExThermoCount; j++)
                            {
                                if (exArgs.ExThermoName[j] == exThermoName) //要求した代用温度計の名前と一致した時
                                {
                                    exThermoValue[i] = exArgs.ExThermoValue[j];
                                    exThermoValue[i] += FanduinaSettings.FicProp[FicNum].ExThermoAdjust[i]; //調整値を加算
                                    isExThermoRequested = true;  //代用温度が取得できたため、送信を要求
                                    break;
                                }
                            }
                        }
                    }
                }
                if (isExThermoRequested)
                {
                    sdArgs = new SendEventArgs();
                    sdArgs.LineNum = 1;
                    sdArgs.SendData[0] = "ExtThm";
                    for (int i = 0; i < TempCount; i++)
                    {
                        sdArgs.SendData[0] += ("," + ((Int32)exThermoValue[i]).ToString());
                    }
                    ficDialog_Send(this, sdArgs);   //送信
                }
            }
            catch
            {
            }
        }
        #endregion

    }

    public class FanduinoRevision   //FanduinoのRevによる機能の違いを表すクラス
    {
        public string Firm
        {
            get { return firm; }
            set
            {
                //Rev,SubVerのチェック
                firm = value.Trim();
                Int32 len = firm.Length;
                if (len > 1)
                {
                    Revision = Convert.ToInt32(firm.Substring(1,1));
                    if (len > 3)
                    {
                        Version = Convert.ToInt32(firm.Substring(3,1));
                    }
                }
                //接続台数の設定
                MaxPWM  = Ver_PWM_Max_chan[Revision];
                MaxFan  = Ver_Fan_Max_chan[Revision];
                MaxTemp = Ver_Thm_Max_chan[Revision];
                //機能の有無の設定
                IsCSFAvalable = false; 
                IsManualAvalable   = false; 
                if (Version >= Ver_Cool_Stop[Revision])
                {
                    IsCSFAvalable = true; 
                }
                if (Version >= Ver_Manual_DT[Revision])
                {
                    IsManualAvalable = true; 
                }
            }
        }
        public Int32  Revision   { get; set; }
        public Int32  Version    { get; set; }
        public Int32  MaxPWM     { get; set; }
        public Int32  MaxFan     { get; set; }
        public Int32  MaxTemp    { get; set; }
        public bool   IsCSFAvalable     { get; set; }
        public bool   IsManualAvalable  { get; set; }
        public FanduinoRevision()
        {
            firm = "";
            Revision = 3;
            Version = 0;
            MaxPWM = 4;
            MaxFan = 6;
            MaxTemp = 4;
            IsCSFAvalable = false;
            IsManualAvalable = false;
        }
 
        //ファームウェアバージョン
        private string firm;
        //リビジョン(Rev)ごとのチャンネル数 (Rev0～)
        //                                Rev.  0   1   2   3   4   5   6   7   8   9
        private Int32[] Ver_PWM_Max_chan = {	0,	4,	4,	4,	4,	4,	6,	4,	2,  6};
        private Int32[] Ver_Fan_Max_chan = {	0,	4,	6,	6,	6,	4,	6,	4,	2,  6};
        private Int32[] Ver_Thm_Max_chan = {	0,	4,	4,	4,	4,	4,	4,	4,	2,  4};
        //バージョン（F*.○）ごとの機能　（機能追加時のバージョン番号　:99は機能なし）
        //                            Rev.   0   1   2   3   4   5   6   7   8   9
        private Int32[] Ver_Cool_Stop = {	99,	99,	99,	 6,	 6,	 6,	 0,	 0,	 0,  0};     //低温時Fan停止機能（CSF)
        private Int32[] Ver_Manual_DT = {	99,	99,	99,	 7,	 7,	 7,	 4,	 0,	 0,  0};     //FanDuty手動設定機能（Manual）
    }

    public class AlarmFanEventArgs : EventArgs  //ファンの警報をMainFormに要求するイベント引数
    {
        public Int32  FicNum    { get; set; }
        public Int32  FanNum    { get; set; }
        public double FanDuty   { get; set; }
        public bool   AlarmByBaloon { get; set; }
        public bool   AlarmBySound  { get; set; }
    }
    public class AlarmTempEventArgs : EventArgs  //温度の警報をMainFormに要求するイベント引数
    {
        public Int32  FicNum    { get; set; }
        public Int32  TempNum   { get; set; }
        public Int32  TempAlarm { get; set; }
        public double TempValue { get; set; }
        public bool   AlarmByBaloon { get; set; }
        public bool   AlarmBySound  { get; set; }
    }

}
