﻿using System;
using System.Data;
using System.Drawing;
using System.IO;
using System.Security.Principal;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace FanduinaV3CS
{
    public partial class FicConfigDialog : Form //ファンコンを設定するダイアログクラス
    {
        #region "ClassDef"
        //プロパティ
        public Int32 FicNum     //登録Fic番号
        {
            get { return ficNum; }
            set { ficNum = value; ficNumber.Text = value.ToString(); }
        }
        public Int32 FanTach    //FicFanParamに表示する回転数
        { get; set; }
        public Int32 FanTachNum //FicFanParamに回転数を表示するファンの番号
        {
            set { FicFanParam[value].FanTach = FanTach; }
        }
        public FicConfigDialogExThermo[] FicExThermo;   //代用温度計のコントロール
        //コンストラクタ
        public FicConfigDialog()
        {
            InitializeComponent();
            Load += FicConfigDialog_Load;
        }
        //イベント宣言
        public event EventHandler<SendEventArgs> SendEvent;    //ファンコンへのデータ送信イベント
        SendEventArgs args = new SendEventArgs();
        //フィールド変数
        private Int32 ficNum;
		private string exeDir;	//このプログラムのexeファイルが格納されているディレクトリ	//V3CSと相違
        //クラス内共通変数・定数
        private const Int32 FicFanMax    =  6;
        private const Int32 FicTempMax   =  4;
        private FanduinoInterface Fic;
        //ロード時初期化
        private void FicConfigDialog_Load( object sender, EventArgs e )
        {
            Fic = (FanduinoInterface)this.Owner;
			exeDir = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location).ToString();	//V3CSと相違
//			MessageBox.Show("FicConfigDialog: " + exeDir );

            #region "AlarmTab init"
            FicTempAlarm = new FicConfigDialogTempAlarm[FicTempMax];
            FicTempAlarm[0] = ficTempAlarm1;
            FicTempAlarm[1] = ficTempAlarm2;
            FicTempAlarm[2] = ficTempAlarm3;
            FicTempAlarm[3] = ficTempAlarm4;
            FicFanAlarm = new CheckBox[FicFanMax];
            FicFanAlarm[0] = ficFanAlarm1;
            FicFanAlarm[1] = ficFanAlarm2;
            FicFanAlarm[2] = ficFanAlarm3;
            FicFanAlarm[3] = ficFanAlarm4;
            FicFanAlarm[4] = ficFanAlarm5;
            FicFanAlarm[5] = ficFanAlarm6;
            RestoreAlarmSettings();
            #endregion

            #region "LogTab init"
            #endregion

            #region "ParamTab init"
            FicFanParam = new FicConfigDialogFanParam[FicFanMax];
            FicFanParam[0] = ficFanParam1;
            FicFanParam[1] = ficFanParam2;
            FicFanParam[2] = ficFanParam3;
            FicFanParam[3] = ficFanParam4;
            FicFanParam[4] = ficFanParam5;
            FicFanParam[5] = ficFanParam6;
            FicTempName = new TextBox[FicTempMax];
            FicTempName[0] = boxTempName1;
            FicTempName[1] = boxTempName2;
            FicTempName[2] = boxTempName3;
            FicTempName[3] = boxTempName4;

            for (int i = 0; i < FicFanMax; i++)
            {
                FicFanParam[i].FicNum = this.FicNum;
                FicFanParam[i].FanNum = i;
                FicFanParam[i].TestModeEvent += TestModeAction;
            }
            ReadAVR();
            waitCount = 0;
            #endregion

            #region "ExThermoTab init"
            FicExThermo = new FicConfigDialogExThermo[FicTempMax];
            FicExThermo[0] = ficExThermo0;
            FicExThermo[1] = ficExThermo1;
            FicExThermo[2] = ficExThermo2;
            FicExThermo[3] = ficExThermo3;
            RestoreExThermoSettings();
            #endregion

            #region "FirmwareTab init"
            #endregion
        }
		//終了
        private void buttonClose_Click( object sender, EventArgs e )    //終了ボタン
        {
			for ( int i = 0; i < FicFanMax; i++ )
			{
				FicFanAlarm[i].Dispose();
				FicFanParam[i].Dispose();
			}
			for ( int i = 0; i < FicTempMax; i++ )
			{
				FicTempAlarm[i].Dispose();
				FicTempName[i].Dispose();
				FicExThermo[i].Dispose();
			}
            this.Close();
        }
        #endregion

        #region "Alarm"     アラーム・タブ
        //変数
        private FicConfigDialogTempAlarm[] FicTempAlarm;
        private CheckBox[] FicFanAlarm;
        //イベント処理
        private void buttonAlarmCancel_Click(object sender, EventArgs e)    //Undoボタン
        {
            RestoreAlarmSettings();
        }
        private void RestoreAlarmSettings()                                 //設定の復元
        {
            for (int i = 0; i < FicTempMax; i++)
            {
                FicTempAlarm[i].IsAlarmEnable = FanduinaSettings.FicProp[FicNum].AlarmTempEnable[i];
                FicTempAlarm[i].AlarmTemp     = FanduinaSettings.FicProp[FicNum].AlarmTempValue[i];
                FicTempAlarm[i].TempName      = Fic.TempParam[i].TempName;
            }
            for (int i = 0; i < FicFanMax; i++)
            {
                FicFanAlarm[i].Checked = FanduinaSettings.FicProp[FicNum].AlarmFanEnable[i];
                FicFanAlarm[i].Text    = Fic.FanParam[i].FanName;
            }
            ficTempAlarmSecond.Value = Convert.ToDecimal(FanduinaSettings.FicProp[FicNum].AlarmTempTimer);
            ficAlarmBalloon.Checked = FanduinaSettings.FicProp[FicNum].AlarmByBalloon;
            ficAlarmSound.Checked   = FanduinaSettings.FicProp[FicNum].AlarmBySound;
        }
        private void buttonAlarmApply_Click(object sender, EventArgs e)     //Applyボタン
        {
            ApplyAlarmSettings();
        }
        private void ApplyAlarmSettings()                                   //設定の適用
        {
            for (int i = 0; i < FicTempMax; i++)
            {
                FanduinaSettings.FicProp[FicNum].AlarmTempEnable[i] = FicTempAlarm[i].IsAlarmEnable;
                FanduinaSettings.FicProp[FicNum].AlarmTempValue[i]  = FicTempAlarm[i].AlarmTemp;
                FicTempAlarm[i].TempName      = Fic.TempParam[i].TempName;
            }
            for (int i = 0; i < FicFanMax; i++)
            {
                FanduinaSettings.FicProp[FicNum].AlarmFanEnable[i] = FicFanAlarm[i].Checked;
                FicFanAlarm[i].Text    = Fic.FanParam[i].FanName;
            }
            FanduinaSettings.FicProp[FicNum].AlarmTempTimer = Convert.ToInt32(ficTempAlarmSecond.Value);
            FanduinaSettings.FicProp[FicNum].AlarmByBalloon = ficAlarmBalloon.Checked;
            FanduinaSettings.FicProp[FicNum].AlarmBySound   = ficAlarmSound.Checked;
        }
        #endregion

        #region "Log"       ログ・タブ
        private void  ShowLogTab()
        {
			try
			{
            	this.logEnable.Checked = FanduinaSettings.FicProp[FicNum].LogEnable;
            	this.logIntervalSecond.Value = Convert.ToDecimal(FanduinaSettings.FicProp[FicNum].LogInterval);
            	this.logFileSize.Text = Convert.ToString(10378 / Convert.ToInt32(logIntervalSecond.Value));
            	showCurrentLog_Click(this, null);
			}
			catch
			{
			}
        }
        private void  buttonLogApply_Click(object sender, EventArgs e)   //ログ設定の保存
        {
            FanduinaSettings.FicProp[FicNum].LogEnable = this.logEnable.Checked;
            FanduinaSettings.FicProp[FicNum].LogInterval = Convert.ToInt32(this.logIntervalSecond.Value);
        }
        private void  logIntervalSecond_ValueChanged(object sender, EventArgs e) //1日あたりのファイルサイズ計算
        {
            logFileSize.Text = Convert.ToString(10378 / Convert.ToInt32(logIntervalSecond.Value));
        }
        private void  buttonBrowse_Click(object sender, EventArgs e)     //過去ログファイルの選択
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Path.Combine(exeDir,"LogFiles");
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                previousLogFile.Text = Path.GetFileName(ofd.FileName);
            }
            ofd.Dispose();
        }
        private void  showCurrentLog_CheckedChanged(object sender, EventArgs e)
        {
            showCurrentLog_Click(sender, e);
        }
        private void  showCurrentLog_Click(object sender, EventArgs e)
        {
            if (showCurrentLog.Checked)
            {
                string filePath = CurrentLogFile();
                LogRead(filePath);
            }
        }
        private void  showPreviousLog_Click(object sender, EventArgs e)
        {
            if (showPreviousLog.Checked)
            {
                string filePath = Path.Combine(exeDir,"LogFiles") + "/" + previousLogFile.Text;	//V3CSと相違
                LogRead(filePath);
            }

        }
        private void  LogRead(string filePath)
        {
            DataTable dt  = new DataTable();
            DataRow   row;
            string[]  stArray = new string[18];

            //フィールド(列)を追加
            dt.Columns.Add("hh:mm:ss", Type.GetType("System.String"));
            for (int i = 0; i < Fic.TempCount; i++)
            {
                dt.Columns.Add("Temp" + (i+1).ToString(), Type.GetType("System.String"));
            }
            for (int i = 0; i < Fic.FanCount; i++)
            {
                dt.Columns.Add("Fan"  + (i+1).ToString(), Type.GetType("System.String"));
            }
            for (int i = 0; i < Fic.FanCount; i++)
            {
                dt.Columns.Add("Duty" + (i+1).ToString(), Type.GetType("System.String"));
            }
            Int32 columMax = Fic.TempCount + Fic.FanCount * 2 + 1;

            //カンマ区切りテキストを読み込み、データテーブルに追加
            Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
            try
            {
                StreamReader LogFileRead = new StreamReader(filePath, sjisEnc);
                while (LogFileRead.Peek() >= 0)
                {
                    string logLine = LogFileRead.ReadLine();
                    stArray = logLine.Replace(",", "\n").Split('\n');
					row = dt.NewRow();
					row.ItemArray = stArray;
                    dt.Rows.Add(row);
                }
                LogFileRead.Close();

				//データグリッドビューにセット
                logDataGridView.DataSource = dt;

                //DataGridViewのセルスタイル設定
                logDataGridView.AllowUserToAddRows = false;
                logDataGridView.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
                logDataGridView.ColumnHeadersDefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
                logDataGridView.RowHeadersWidth = 20;
                logDataGridView.Columns[0].Frozen = true;
                logDataGridView.Rows[0].Frozen = true;
                logDataGridView.Columns[0].Width = 80;
                for (int i = 1; i < columMax; i++)
                {
                    logDataGridView.Columns[i].Width = 50;
                }
                foreach (DataGridViewColumn c in logDataGridView.Columns)
                {
                    c.SortMode = DataGridViewColumnSortMode.NotSortable;
                }
                dt.Dispose();
            }
            catch
            {
//                MessageBox.Show(Properties.Resources.ErrLogRead,Properties.Resources.ErrorMessage,
//                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private string  CurrentLogFile()
        {
            string dirPath = Path.Combine(exeDir,"LogFiles") ;	//V3CSと相違
            string startDateTime = DateTime.Now.ToString("yyyy-MM-dd");
            string currentLogFileName = "Log-" + startDateTime + "-" + FicNum.ToString().Trim() + "-" + Fic.Firm.Trim() + ".txt";
            currentLogFile.Text = currentLogFileName;
            string filePath = dirPath + "/" + currentLogFileName;	//V3CSと相違
            return filePath;
        }

        #endregion

        #region "Parameter" パラメータ・タブ
        //変数
        private FicConfigDialogFanParam[] FicFanParam;
        private TextBox[] FicTempName;
        private Int32 waitCount = 0;
        private const Int32 WAITCOUNT_MAX  =  3;
        private const Int32 TESTMODE_ENTER =  1;
        private const Int32 TESTMODE_EXIT  = -1;
        private const Int32 TESTMODE_EXEC  =  0;
        //イベント処理
        private void TestModeAction(object sender, TestModeEventArgs e) //FanParamからのテストモードイベントを処理
        {
            if (e.TestMode == TESTMODE_ENTER)
            {
                //SendEventArgs args = new SendEventArgs();
                args.LineNum = 1;
                args.SendData[0] = "TestOn, " + (e.FanNum+1).ToString();
                SendEvent(this, args);
            }
            else if (e.TestMode == TESTMODE_EXEC)
            {
                if (waitCount == 0)
                {
                    labelDman.ForeColor = Color.Red;
                    waitCount = WAITCOUNT_MAX;
                    SendEventArgs args = new SendEventArgs();
                    args.LineNum = 1;
                    args.SendData[0] = "Test, " + (e.FanNum+1).ToString() +
                                       ", "    + e.Duty.ToString();
                    SendEvent(this, args);
                }
                else
                {
                    labelDman.ForeColor = Color.Black;
                    waitCount--;
                }
            }
            else if (e.TestMode == TESTMODE_EXIT)
            {
                labelDman.ForeColor = Color.Black;
                SendEventArgs args = new SendEventArgs();
                args.LineNum = 1;
                args.SendData[0] = "TestOff, " + (e.FanNum + 1).ToString();
                for (int i = 0; i < 10; i++)
                {
                    SendEvent(this, args);
                    System.Threading.Thread.Sleep(300);
                    if (!Fic.IsTestModeOn) break;
                }
            }
 
        }
        private void buttonReadAVR_Click(object sender, EventArgs e)    //パラメータ読込み
        {
            ReadAVR();
        }
        private void ReadAVR()                                          //AVRから読込み
        {
            for (int i = 0; i < 6; i++)
            {
                if (Fic.IsCSFAvalable) { FicFanParam[i].IsCSFAvailable = true ; }
                else                          { FicFanParam[i].IsCSFAvailable = false; }

                if (Fic.IsManualAvalable) { FicFanParam[i].IsManualAvailable = true ; }
                else                             { FicFanParam[i].IsManualAvailable = false; }
                if (i < Fic.PwmCount) { FicFanParam[i].IsPWMAvailable = true ; }
                else
                {
                    FicFanParam[i].IsPWMAvailable    = false; 
                    FicFanParam[i].IsCSFAvailable    = false; 
                }
                if (i < Fic.FanCount) { FicFanParam[i].IsFanAvailable = true ; }
                else
                {
                    FicFanParam[i].IsFanAvailable    = false; 
                    FicFanParam[i].IsCSFAvailable    = false; 
                    FicFanParam[i].IsManualAvailable = false;  
                }
                FicFanParam[i].FanParams = Fic.FanParam[i];
            }
            for (int i = 0; i < 4; i++)
            {
                FicTempName[i].Text = Fic.TempParam[i].TempName;
            }
        }
        private void buttonWriteAVR_Click(object sender, EventArgs e)   //パラメータ書込み
        {
            WriteAVR();
        }
        private void WriteAVR()                                         //AVRに書込み
        {
            //シリアル変換
            bool result = ParamToLine(this,args);
            //送信要求イベント
            if (result)
            {
                SendEvent(this, args);
                if (args.Success)
                {
                    MessageBox.Show(Properties.Resources.MsgParamWrite, Properties.Resources.InformationMessage,
                                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }
        private void buttonExport_Click(object sender, EventArgs e)     //ファイルにエクスポート
        {
            //パラメータを変換
            if (!ParamToLine(this,args))
            {
		        return; //パラメータにエラーがあるときは保存しない
	        }
 	        SaveFileDialog sfd = new SaveFileDialog();
	        sfd.FileName = "AVR-Fancon_Parameter" + FicNum.ToString() + ".txt";
            sfd.InitialDirectory = Path.Combine(exeDir ,"Settings");

	        if (sfd.ShowDialog() == DialogResult.OK)
            {
                Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
                StreamWriter ParamFileWrite = new StreamWriter(sfd.FileName, false, sjisEnc);
		        for (int i = 0; i < args.LineNum; i++)
                {
			        string strDataSend = args.SendData[i];
			        ParamFileWrite.WriteLine(strDataSend);
		        }
		        ParamFileWrite.Close();   //ファイルクルーズ
                MessageBox.Show(Properties.Resources.MsgParamExport, Properties.Resources.InformationMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
	        }
	        sfd.Dispose();
        }
        private bool ParamToLine(object sender, SendEventArgs e)        //パラメータをシリアルに変換
        {
            e.LineNum = 0;
            e.SendData[e.LineNum] = "SetPrm,FanNam";
            for ( Int32 i = 0; i < Fic.FanCount; i++ )
            {
                if (!Regex.IsMatch(FicFanParam[i].FanParams.FanName, @"^[a-zA-Z0-9\s]{1,6}$"))
                {
                    MessageBox.Show(Properties.Resources.ErrFanName, Properties.Resources.ErrorMessage, 
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.FanName;
                }
            }

            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,ThmNam";
            for (Int32 i = 0; i < Fic.TempCount; i++)
            {
                if (!Regex.IsMatch(FicTempName[i].Text, @"^[a-zA-Z0-9\s]{1,6}$"))
                {
                    MessageBox.Show(Properties.Resources.ErrTempName, Properties.Resources.ErrorMessage, 
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    e.SendData[e.LineNum] += "," + FicTempName[i].Text;
                }
            }

            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,Tmin";
            for ( Int32 i = 0; i < Fic.PwmCount; i++ )
            {
                if (FicFanParam[i].FanParams.Tmin >= FicFanParam[i].FanParams.Tmax)
                {
                    MessageBox.Show(Properties.Resources.ErrTminTmax, Properties.Resources.ErrorMessage, 
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                else
                {
                    e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Tmin.ToString();
                }
            }

            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,Tmax";
            for ( Int32 i = 0; i < Fic.PwmCount; i++ )
            {
                e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Tmax.ToString();
            }
 
            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,Dmin";
            for ( Int32 i = 0; i < Fic.PwmCount; i++ )
            {
                e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Dmin.ToString();
            }
 
            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,Dmax";
            for ( Int32 i = 0; i < Fic.PwmCount; i++ )
            {
                e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Dmax.ToString();
            }
 
            e.LineNum += 1;
            e.SendData[e.LineNum] = "SetPrm,FanThm";
            for ( Int32 i = 0; i < Fic.PwmCount; i++ )
            {
                e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.FanThm.ToString();
            }
            //低温時Fan停止機能(CSF) Tstop
            if (Fic.IsCSFAvalable)
            {
                e.LineNum += 1;
                e.SendData[e.LineNum] = "SetPrm,Tstop";
                for (Int32 i = 0; i < Fic.PwmCount; i++)
                {
                    if (FicFanParam[i].FanParams.Tstop >= FicFanParam[i].FanParams.Tmin)
                    {
                        MessageBox.Show(Properties.Resources.ErrTstopTmin, Properties.Resources.ErrorMessage, 
                                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return false;
                    }
                    else
                    {
                        e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Tstop.ToString();
                    }
                }
            }
            //手動設定
            if (Fic.IsManualAvalable)
            {
                e.LineNum += 1;
                e.SendData[e.LineNum] = "SetPrm,Dman";
                for (Int32 i = 0; i < Fic.PwmCount; i++)
                {
                    e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.Dman.ToString();
                }
                e.LineNum += 1;
                e.SendData[e.LineNum] = "SetPrm,FanMan";
                for (Int32 i = 0; i < Fic.PwmCount; i++)
                {
                    e.SendData[e.LineNum] += "," + FicFanParam[i].FanParams.FanMan.ToString();
                }
                e.LineNum += 1;
            }
            return true;
        }
        private void buttonImport_Click(object sender, EventArgs e)     //ファイルからインポート
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = Path.Combine(exeDir ,"Settings");
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                Encoding sjisEnc = Encoding.GetEncoding("Shift_JIS");
                StreamReader ParamFileRead = new StreamReader(ofd.FileName, sjisEnc);
                args.LineNum = 0;
                while (ParamFileRead.Peek() >= 0)
                {
                    args.SendData[args.LineNum] = ParamFileRead.ReadLine();
                    args.LineNum += 1;
                }
                ParamFileRead.Close();
                LineToParam(this, args);
                MessageBox.Show(Properties.Resources.MsgParamImport, Properties.Resources.InformationMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            ofd.Dispose();
        }
        private void LineToParam(object sender, SendEventArgs e)        //シリアルからパラメータに変換
        {

            FanParamStruct[] param = new FanParamStruct[FicFanMax];
            string reportItem = null;
            string reportItemSetting = null;
            string[] stArray = new string[13];
            Int32 maxPWM  = Fic.PwmCount;
            Int32 maxFan  = Fic.FanCount;
            Int32 maxTemp = Fic.TempCount;
            Int32 dmyFan = maxFan - maxPWM;

            //受信データの表示
            for (int i = 0; i < e.LineNum; i++)
            {
                try
                {
                    stArray = e.SendData[i].Replace(",", "\n").Split('\n');
                }
                catch
                {
                }
                if (stArray.Length > 1)
                {
                    reportItem = stArray[0].Trim();
                    if (reportItem == "SetPrm")
                    {
                        try
                        {
                            reportItemSetting = stArray[1].Trim();
                            switch (reportItemSetting)
                            {
                                case "Tmin": //ファン制御最低温度（℃）                  
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Tmin = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Tmin = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "Tmax": //ファン制御最高温度（℃）
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Tmax = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Tmax = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "Tstop": //ファン停止温度（℃）
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Tstop = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Tstop = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "Dmin": //ファン制御最低温度でのDuty比（％）
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Dmin = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Dmin = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "Dmax": //ファン制御最高温度でのDuty比（％）
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Dmax = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Dmax = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "Dman": //ファン手動制御時のDuty比（％）
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].Dman = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].Dman = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "FanThm": //ファン制御を行う温度計番号
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].FanThm = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].FanThm = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "FanMan": //ファン自動／手動設定
                                    for (int fan = 0; fan < maxPWM; fan++) { param[fan].FanMan = Convert.ToInt32(stArray[fan + 2]); }
                                    if (dmyFan > 0)
                                    {
                                        for (int fan = maxPWM; fan <= maxFan; fan++) { param[fan].FanMan = Convert.ToInt32(stArray[fan + 2 - dmyFan]); }
                                    }
                                    break;
                                case "FanNam": //ファン名称
                                    for (int fan = 0; fan < maxFan; fan++)
                                    {
                                        string fname = stArray[fan + 2].Trim();
                                        param[fan].FanName = fname.PadRight(6, ' ');
                                    }
                                    break;
                                case "ThmNam": //温度計名称
                                    for (int tmp = 0; tmp < maxTemp; tmp++)
                                    {
                                        string tname = stArray[tmp + 2].Trim();
                                        FicTempName[tmp].Text = tname.PadRight(6, ' ');
                                    }
                                    break;
                            }
                        }
                        catch
                        {
                        }
                    }
                }
            }                  
            for (int fan = 0; fan < maxPWM; fan++)
            {
                FicFanParam[fan].FanParams = param[fan];
            }
        }
        #endregion

        #region "ExThermo"  //代用温度計・タブ
        private void applyExthermo_Click(object sender, EventArgs e)    //設定の保存・適用
        {
            for (int i = 0; i < Fic.TempCount; i++)
            {
                FanduinaSettings.FicProp[FicNum].ExThermoType[i]   = FicExThermo[i].ExThermoType;
                FanduinaSettings.FicProp[FicNum].ExThermoName[i]   = FicExThermo[i].ExThermoName;
                FanduinaSettings.FicProp[FicNum].ExThermoAdjust[i] = FicExThermo[i].ExThermoAdjust;
            }
        }
        private void RestoreExThermoSettings()
        {
            try
            {
                for (int i = 0; i < Fic.TempCount; i++)
                {
                    FicExThermo[i].FicNum = this.FicNum;
                    FicExThermo[i].TempNum = i;
                    FicExThermo[i].ExThermoType = FanduinaSettings.FicProp[FicNum].ExThermoType[i];
                    FicExThermo[i].ExThermoName = FanduinaSettings.FicProp[FicNum].ExThermoName[i];
                    FicExThermo[i].ExThermoAdjust = FanduinaSettings.FicProp[FicNum].ExThermoAdjust[i];
                }
            }
            catch
            {
            }
        }
        private void PrivilegeCheck()               //管理者権限のチェック
        {
                Thread.GetDomain().SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
                WindowsPrincipal currentPrincipal = (WindowsPrincipal)Thread.CurrentPrincipal;
                if (!currentPrincipal.IsInRole(WindowsBuiltInRole.Administrator))
                {
                    MessageBox.Show(Properties.Resources.MsgNeedPrivileges, Properties.Resources.WarningMessage,
                                    MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
        }
        private void ExThermoShow(bool showTemp)    //代用温度の表示更新の有無を設定
        {
            for (int i = 0; i < 4; i++)
            {
                FicExThermo[i].TimerEnable = showTemp;
            }
        }
        #endregion

        #region "Firmware"  //ファーム書込・タブ
        private void runFanduineW1_Click(object sender, EventArgs e)    //FanduineW1起動
        {
                MessageBox.Show(Properties.Resources.MsgFanduineW1, Properties.Resources.InformationMessage, 
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
//            try
//            {
//                Fic.PortClose();
//                System.Diagnostics.Process p = System.Diagnostics.Process.Start("FanduineW1/FanduineW1.exe");
//            }
//            catch
//            {
//                MessageBox.Show(Properties.Resources.ErrStartFanduineW1, Properties.Resources.ErrorMessage, 
//                                MessageBoxButtons.OK, MessageBoxIcon.Error);
//            }
        }
        private void runFlip_Click(object sender, EventArgs e)          //Flip起動
        {
                MessageBox.Show(Properties.Resources.MsgFlip, Properties.Resources.InformationMessage, 
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
//            try
//            {
//                args.LineNum = 1;
//                args.SendData[0] = "BTLDR,mega32U4,loader";
//                SendEvent(this, args);
//                Fic.PortClose();
//            } 
//            catch
//            {
//                MessageBox.Show(Properties.Resources.ErrStartBootloader, Properties.Resources.ErrorMessage, 
//                                MessageBoxButtons.OK, MessageBoxIcon.Error);
//            }
//            try
//            {
//	            System.Diagnostics.Process p = System.Diagnostics.Process.Start("flip.exe");
//            }
//            catch
//            {
//                MessageBox.Show(Properties.Resources.ErrStartFlip, Properties.Resources.ErrorMessage, 
//                                MessageBoxButtons.OK, MessageBoxIcon.Error);
//            }
        }
        private void reconnectFanduino_Click(object sender, EventArgs e)//再接続
        {
            Fic.PortConnect();
        }
        #endregion

        #region "TabChange" //タブ変更時の処理
        private void TabWindow_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (TabWindow.SelectedIndex)
            {
                case 0: RestoreAlarmSettings();      //AlarmTab
                    ExThermoShow(false); break;
                case 1: ShowLogTab();                //LogTab
                    ExThermoShow(false); break;
                case 2: ReadAVR();                   //ParamTab
                    ExThermoShow(false); break;
                case 3: RestoreExThermoSettings();   //ExThermoTab
                    ExThermoShow(true);  break;
                default:
                    ExThermoShow(false); break;
            }
        }
        #endregion

    }

    public class SendEventArgs : EventArgs  //Ficからファンコンにデータを送信するよう要求するイベント引数
    {
        public Int32 LineNum { get; set; }
        public bool Success { get; set; }
        public String[] SendData { get; set; }
        public SendEventArgs()
        {
            LineNum = 1;
            Success = false;
            SendData = new String[20];
        }
    }

}
