﻿using System;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    public partial class FicConfigDialogExThermo : UserControl
    {
        //プロパティ
        public Int32  FicNum            //Ficの登録番号
        { get; set; }
        public Int32  TempNum           //温度計番号
        {
            get
            {
                return tempNum; 
            }
            set 
            {
                tempNum = value;
                this.boxTemp.Text = "Temp" + (value+1).ToString(); 
            }
        }
        public Int32  ExThermoType      //代用温度計のタイプ	//V3CSと相違
        {
            get
            {
                if      (buttonOthers.Checked)    return ExThermo.Others;
                else if (buttonCPU.Checked)       return ExThermo.CPU;
                else if (buttonGeForce.Checked)   return ExThermo.GeForce;
                else if (buttonHDD.Checked)       return ExThermo.HDD;
                else if (buttonRadeon.Checked)    return ExThermo.Radeon;
                else                              return ExThermo.OwnSensor;
            }
            set
            {
                buttonOthers.Checked    = false;
                buttonCPU.Checked       = false; 
                buttonGeForce.Checked   = false;
                buttonHDD.Checked       = false;
                buttonRadeon.Checked    = false;
                buttonOwnSensor.Checked = false;
                switch (value)
                {
                    case ExThermo.Others   : buttonOthers.Checked   = true; break;
                    case ExThermo.CPU      : buttonCPU.Checked      = true; break;
                    case ExThermo.GeForce  : buttonGeForce.Checked  = true; break;
                    case ExThermo.HDD      : buttonHDD.Checked      = true; break;
                    case ExThermo.Radeon   : buttonRadeon.Checked   = true; break;
                    default: buttonOwnSensor.Checked = true; break;
                }
            }
        }
        public Int32  ExThermoAdjust    //代用温度計の補正値
        {
            get { return Convert.ToInt32(exThermoAdjust.Value); }
            set { exThermoAdjust.Value = Convert.ToDecimal(value); }
        }
        public Double ExThermoDisp      //表示する代用温度（補正後）
        {
            get { return Convert.ToDouble(exThermoDisp.Text); }
            set
            {
                exThermoDisp.Text = value.ToString("0.0");
            }
        }
        public String ExThermoName      //代用温度計の名称
        {
            get
            {
                return exThermoNameString;
            }
            set
            {
                exThermoNameString = value;
            }
        }
        public bool   TimerEnable       //表示用タイマーの起動・停止  
        {
            get { return timerTempGet.Enabled; }
            set { timerTempGet.Enabled = value; }
        }
        //コンストラクタ
        public FicConfigDialogExThermo()
        {
            InitializeComponent();
            Load += FicConfigDialogExThermo_Load;
        }
        //フィールド変数
        private Int32 tempNum;
        private string[] exThermoNames = new string[20];
        private ExThermoEnumArgs args;
        private string exThermoNameString;
        //ロード時初期化
        private void FicConfigDialogExThermo_Load(object sender, EventArgs e )
        {
            exThermoNameString = Properties.Resources.ExThermoNoSubs;
            exThermoName.Text = exThermoNameString; 
        }
        //クリック時の処理
        private void buttonOwnSensor_Click(object sender, EventArgs e)
        {
            if (buttonOwnSensor.Checked == true) exThermoName.Text = Properties.Resources.ExThermoNoSubs;
        }
        private void buttonExternal_CheckedChanged(object sender, EventArgs e)
        {
            if (buttonOthers.Checked == true) exThermoName.Text = Properties.Resources.ExThermoPleaseSelect;
        }
        private void buttonCPU_CheckedChanged(object sender, EventArgs e)
        {
            if (buttonCPU.Checked == true) exThermoName.Text = Properties.Resources.ExThermoPleaseSelect;
        }
        private void buttonGeForce_CheckedChanged(object sender, EventArgs e)	//V3CSと相違
        {
            if (buttonGeForce.Checked == true) exThermoName.Text = Properties.Resources.ExThermoPleaseSelect;
        }
        private void buttonHDD_CheckedChanged(object sender, EventArgs e)
        {
            if (buttonHDD.Checked == true) exThermoName.Text = Properties.Resources.ExThermoPleaseSelect;
        }
        private void buttonRadeon_CheckedChanged(object sender, EventArgs e)	//V3CSと相違
        {
            if (buttonRadeon.Checked == true) exThermoName.Text = Properties.Resources.ExThermoPleaseSelect;
        }
        private void exTempName_Click(object sender, EventArgs e) //ComboBoxのリストを要求
        {
            try
            {
                exThermoName.Items.Clear();
                if (ExThermoType != ExThermo.OwnSensor)
                {
                    args = new ExThermoEnumArgs(this.ExThermoType);
                    args.FicNum = this.FicNum;
                    args.TempNum = this.TempNum;
                    bool rapid = true;
                    ExThermoClass.ExThermoEnumRequest(rapid, ref args);
                    if (args.Success)
                    {
                        for (int i = 0; i < args.ExThermoCount; i++)
                        {
                            exThermoNames[i] = args.ExThermoName[i];
                            string exNameTemp = args.ExThermoName[i] + " :" + (args.ExThermoValue[i]).ToString("0.0") + "°C";
                            exThermoName.Items.Add(exNameTemp);
                        }
                    }
                }
            }
            catch
            {
            }
        }
        private void exThermoName_SelectionChangeCommitted(object sender, EventArgs e)  //Textには温度表示抜きのStringを格納
        {
            if (exThermoName.SelectedIndex >= 0)
			{
				exThermoNameString = exThermoNames[exThermoName.SelectedIndex];
            	exThermoName_Leave(sender, e);
			}
        }
        private void exThermoName_Leave(object sender, EventArgs e) //プルダウンから離れた時は、選択されている代用温度の名称のみを表示
        {
            exThermoName.Items.Clear();
            exThermoName.Items.Add(exThermoNameString);
            exThermoName.SelectedIndex = 0;
            exThermoName.SelectedItem = exThermoNameString;
            exThermoName.Text = exThermoNameString;
        }
        private void timerTempGet_Tick(object sender, EventArgs e)  //タイマーによる表示温度の更新
        {
            try
            {
                if (ExThermoType != ExThermo.OwnSensor)
                {
                    args = new ExThermoEnumArgs(this.ExThermoType);
                    args.FicNum = this.FicNum;
                    args.TempNum = this.TempNum;
                    bool rapid = false;
                    ExThermoClass.ExThermoEnumRequest(rapid, ref args);
                    if (args.Success)
                    {
                        for (int i = 0; i < args.ExThermoCount; i++)
                        {
                            if (args.ExThermoName[i] == ExThermoName)
                            {
                                ExThermoDisp = args.ExThermoValue[i] + (double)ExThermoAdjust;
                                break;
                            }
                        }
                    }
                    else
                    {
                        ExThermoDisp = 0.0;
                    }
                }
                else
                {
                    ExThermoDisp = 0.0;
                }
            }
            catch
            {
            }
        }
    }

}
