﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    public partial class FicConfigDialogFanParam : UserControl  //ファン・パラメータ設定・表示用のクラス
    {
        #region "ClassDef"
        //プロパティ
        public Int32 FicNum
        {
            get { return fanParams.FicNum;  }
            set { fanParams.FicNum = value; }
        }
        public Int32 FanNum
        {
            get { return fanParams.FanNum;  }
            set { fanParams.FanNum = value; }
        }
        public Int32 FanTach { set { fanTach.Text = value.ToString(); } }
        public FanParamStruct FanParams
        {
            get
            {
                FanParamsGet();
                return fanParams;
            }
            set
            {
                fanParams  = value;
                FanParamsSet();
            }
        }
        public bool IsFanAvailable
        {
            get { return isFanAvailable; }
            set
            {
                isFanAvailable = value;
                if (value)
                {
                    EnableFan();
                }
                else
                {
                    DisableFan();
                }
            }
        }
        public bool IsPWMAvailable
        {
            get { return isPWMAvailable; }
            set
            {
                isPWMAvailable = value;
                if (value)
                {
                    EnablePWM();
                }
                else
                {
                    DisablePWM();
                }
            }
        }
        public bool IsCSFAvailable
        {
            get { return isCSFAvaiable; }
            set
            {
                isCSFAvaiable = value;
                if (value)
                {
                    EnableCSF();
                }
                else
                {
                    DisableCSF();
                }
            }
        }
        public bool IsManualAvailable
        {
            get { return isManualAvailable; }
            set
            {
                isManualAvailable = value;
                if (value)
                {
                    EnableManual();
                }
                else
                {
                    DisableManual();
                }
            }
        }
        public bool IsTestModeOn
        {
            get { return isTestModeOn; }
            set
            {
                isTestModeOn = value;
                if (value)
                {
                   barDman.BackColor    = manualColor;
                }
                else
                {
                    barDman.BackColor    = autoColor;
                }
            }
        }
        //コンストラクタ
        public FicConfigDialogFanParam()
        {
            InitializeComponent();
            Load += FicConfigDialogFanParam_Load;
        }
        //フィールド変数
        private FanParamStruct  fanParams;
        private bool isFanAvailable    = true;
        private bool isPWMAvailable    = true;
        private bool isCSFAvaiable     = true;
        private bool isManualAvailable = true;
        private bool isTestModeOn      = false;
        //ロード時初期化
        private void FicConfigDialogFanParam_Load(object sender, EventArgs e)
        {
            FanParams = new FanParamStruct(0, 0);
        }
        //背景色の設定
        private Color foreColor    = Color.Black;
        private Color nameColor    = Color.White;
        private Color tempColor    = Color.FromArgb(255, 255, 192, 192);
        private Color dutyColor    = Color.FromArgb(255, 255, 255, 192);
        private Color stopColor    = Color.FromArgb(255, 192, 255, 255);
        private Color manualColor  = Color.FromArgb(255, 255, 224, 192);
        private Color autoColor    = Color.DarkGray;
        private Color thmColor     = Color.FromArgb(255, 192, 255, 192);
        private Color disableColor = Color.Gray;    //使えない時は灰色にする
        //パラメータデータの受け渡し処理
        private void FanParamsGet()
        {
            if (isFanAvailable)
            {
                fanParams.FanName = boxFanName.Text.PadRight(6, ' ');
            }
            else
            {
                fanParams.FanName = "      ";
            }
            if (isPWMAvailable)
            {
                fanParams.Tmin = Convert.ToInt32(boxTmin.Text);
                fanParams.Tmax = Convert.ToInt32(boxTmax.Text);
                fanParams.Dmin = Convert.ToInt32(boxDmin.Text);
                fanParams.Dmax = Convert.ToInt32(boxDmax.Text);
                fanParams.FanThm = Convert.ToInt32(boxFanThm.Value);
            }
            else
            {
                fanParams.Tmin = 0;
                fanParams.Tmax = 99;
                fanParams.Dmin = 100;
                fanParams.Dmax = 100;
                fanParams.FanThm = 1;
            }
            if (isCSFAvaiable)
            {
                fanParams.Tstop = Convert.ToInt32(boxTstop.Text);
            }
            else
            {
                fanParams.Tstop = 0;
            }
            if (isManualAvailable)
            {
                fanParams.Dman = Convert.ToInt32(boxDman.Text);
                fanParams.FanMan = 0;
                if (boxFanMan.Checked) fanParams.FanMan = 1;
            }
            else
            {
                fanParams.Dman = 100;
                fanParams.FanMan = 0;
            }
        }
        private void FanParamsSet()
        {
            if (isFanAvailable)
            {
                boxFanName.Text = fanParams.FanName;
            }
            if (isPWMAvailable)
            {
                boxTmin.Text = fanParams.Tmin.ToString();
                boxTmax.Text = fanParams.Tmax.ToString();
                boxDmin.Text = fanParams.Dmin.ToString();
                boxDmax.Text = fanParams.Dmax.ToString();
                boxFanThm.Value = Convert.ToDecimal(fanParams.FanThm);
            }
            if (isCSFAvaiable)
            {
                boxTstop.Text = fanParams.Tstop.ToString();
            }
            if (isManualAvailable)
            {
                boxDman.Text = fanParams.Dman.ToString();
                barDman.Value = fanParams.Dman;
                boxFanMan.Checked = false;
                if (fanParams.FanMan != 0) boxFanMan.Checked = true;
            }
        }
        //表示設定処理
        private void EnableFan()
        {
            boxFanName.BackColor  = nameColor;
            boxFanName.Enabled    = true;
        }
        private void DisableFan()
        {
            boxFanName.BackColor  = disableColor;
            boxFanName.Enabled    = false;
        }
        private void EnablePWM()
        {
            boxTmin.BackColor = tempColor;
            boxDmin.BackColor = dutyColor;
            boxTmax.BackColor = tempColor;
            boxDmax.BackColor = dutyColor;
            boxFanThm.ForeColor = foreColor;
            boxFanThm.BackColor = thmColor;
            boxTmin.Enabled = true;
            boxDmin.Enabled = true;
            boxTmax.Enabled = true;
            boxDmax.Enabled = true;
            boxFanThm.Enabled = true;
        }
        private void DisablePWM()
        {
            boxTmin.BackColor = disableColor;
            boxDmin.BackColor = disableColor;
            boxTmax.BackColor = disableColor;
            boxDmax.BackColor = disableColor;
            boxFanThm.ForeColor = disableColor;
            boxFanThm.BackColor = disableColor;
            boxTmin.Enabled = false;
            boxDmin.Enabled = false;
            boxTmax.Enabled = false;
            boxDmax.Enabled = false;
            boxFanThm.Enabled = false;
            boxFanMan.Enabled = false;
        }
        private void EnableCSF()
        {
            boxTstop.BackColor = stopColor;
            boxTstop.Enabled = true;
        }
        private void DisableCSF()
        {
            boxTstop.BackColor = disableColor;
            boxTstop.Enabled = false;
        }
        private void EnableManual()
        {
            boxDman.BackColor = manualColor;
            boxDman.Enabled = true;
            barDman.Enabled = true;
            if (isPWMAvailable) {boxFanMan.Enabled = true;}
                           else {boxFanMan.Enabled = false;}
        }
        private void DisableManual()
        {
            boxDman.BackColor = disableColor;
            boxDman.Enabled = false;
            boxFanMan.Enabled = false;
            barDman.Enabled = false;
        }
        #endregion

        #region "CheckData" //入力チェック
        private bool NumCheck(string text)                                      //入力が0～100の数値かどうかをチェック 数値であればtrueを返す
        {
            try
            {
                Int32 i = Convert.ToInt32(text);
                if (i < 0 | i > 100)
                {
                    MessageBox.Show(Properties.Resources.ErrInputValue, Properties.Resources.ErrorMessage, 
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return true;
                }
                else
                {
                    return true;
                }
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrInputNumeric, Properties.Resources.ErrorMessage, 
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
        }
        private void boxFanName_Validating(object sender, CancelEventArgs e)    //FanNameは1～6文字
        {
            boxFanName.Text = boxFanName.Text.Trim();
            Int32 i = boxFanName.TextLength;
            if (i < 1 | i > 6)
            {
                MessageBox.Show(Properties.Resources.ErrFanName, Properties.Resources.ErrorMessage, 
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void boxFanName_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxFanName_Validating(this, null);
        }
        private void boxTmin_Validating(object sender, CancelEventArgs e)
        {
            if ( !NumCheck(boxTmin.Text) ) boxTmin.Text = "0";
        }
        private void boxTmin_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxTmin_Validating(this, null);
        }
        private void boxDmin_Validating(object sender, CancelEventArgs e)
        {
            if (!NumCheck(boxDmin.Text)) boxDmin.Text = "100";
        }
        private void boxDmin_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxDmin_Validating(this, null);
        }
        private void boxTmax_Validating(object sender, CancelEventArgs e)
        {
            if (!NumCheck(boxTmax.Text)) boxTmax.Text = "99";
        }
        private void boxTmax_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxTmax_Validating(this, null);
        }
        private void boxDmax_Validating(object sender, CancelEventArgs e)
        {
            if (!NumCheck(boxDmax.Text)) boxDmax.Text = "100";
        }
        private void boxDmax_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxDmax_Validating(this, null);
        }
        private void boxTstop_Validating(object sender, CancelEventArgs e)
        {
            try
            {
                Int32 i = Convert.ToInt32(boxTstop.Text);
                if (i < -250 | i > 99)
                {
                    MessageBox.Show(Properties.Resources.ErrTstopValue, Properties.Resources.ErrorMessage, 
                                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrInputNumeric, Properties.Resources.ErrorMessage, 
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
                boxTstop.Text = "0";
            }
        }
        private void boxTstop_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxTstop_Validating(this, null);
        }
        private void boxDman_Validating(object sender, CancelEventArgs e)
        {
            {
                if (!NumCheck(boxDman.Text))
                {
                    boxDman.Text = "100";
                }
                barDman.Value = Convert.ToInt32(boxDman.Text);
            }
        }
        private void boxDman_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter) boxDman_Validating(this, null);
        }
        #endregion

        #region "TestMode" //テストモード
        public event EventHandler<TestModeEventArgs> TestModeEvent;  
        private const Int32 TESTMODE_ENTER =  1;    //テストモード開始
        private const Int32 TESTMODE_EXIT  = -1;    //テストモード実施中
        private const Int32 TESTMODE_EXEC  =  0;    //テストモード終了
        //テストモードのきっかけとなるイベント
        private void barDman_Enter(object sender, EventArgs e)
        {
            EnterTestMode();
        }
        private void barDman_MouseLeave(object sender, EventArgs e)
        {
            ExitTestMode();
        }
        private void barDman_MouseDown(object sender, MouseEventArgs e)
        {
            EnterTestMode();
        }
        private void timer1_Tick(object sender, EventArgs e)
        {
            boxDman.Text = barDman.Value.ToString(); 
            ExecTestMode();
        }
        //FicSettingsDialogにテストモードイベントを通知
        private void EnterTestMode()
        {
            if (this.IsTestModeOn == false)
            {
                this.IsTestModeOn = true;
                timer1.Start();
                TestModeEventArgs args = new TestModeEventArgs();
                args.TestMode = TESTMODE_ENTER;
                args.FanNum   = FanNum;
                args.Duty     = barDman.Value;
                TestModeEvent(this, args);
            }
        }
        private void ExecTestMode()
        {
            if (this.IsTestModeOn == true)
            {
                TestModeEventArgs args = new TestModeEventArgs();
                args.TestMode = TESTMODE_EXEC;
                args.FanNum = FanNum;
                args.Duty = barDman.Value;
                TestModeEvent(this, args);
            }
        }
        private void ExitTestMode()
        {
            if (this.IsTestModeOn == true)
            {
                this.IsTestModeOn = false;
                timer1.Stop();
                TestModeEventArgs args = new TestModeEventArgs();
                args.TestMode = TESTMODE_EXIT;
                args.FanNum   = FanNum;
                args.Duty     = barDman.Value;
                TestModeEvent(this, args);
            }
        }
        #endregion
    }

    public class TestModeEventArgs : EventArgs  //テストモードの状態を伝えるイベント引数
    {
        public Int32 TestMode { get; set; }
        public Int32 FanNum   { get; set; }
        public Int32 Duty     { get; set; }
    }
}
