﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Management;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Xml.Serialization;
using System.Diagnostics;	//V3CSと相違

//using GetCoreTempInfoNET;	//V3CSと相違
//using FanduinaAtiTemp;	//V3CSと相違
//using FanduinaSFTemp;	//V3CSと相違

namespace FanduinaV3CS
{
    public partial class MainForm : Form
    {
        #region Application Start 開始時の処理
        //コンストラクタ
        public MainForm()
        {
            InitializeComponent();
            Load += MainForm_Load;
        }
        //共通で使用するクラス
        static public FanduinaSettings  fanduinaSettings = new FanduinaSettings();  //設定値の保存用クラス
        static public FanduinoInterface[] Fic =new FanduinoInterface[4];            //ファンコン用インターフェース
        static public SidePanel         SidePanel;                					  //モニタ用サイドパネル
		//フィールド変数
		private string exeDir;	//このプログラムのexeファイルが格納されているディレクトリ	//V3CSと相違
		//メインフォームをロード
        private void MainForm_Load(object sender, EventArgs e)
        {
			exeDir = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location).ToString();	//V3CSと相違
            //起動時にMainFormを非表示化
//            {
//                Application.DoEvents();   //V3CSと相違
//                ShowInTaskbar = false;	//V3CSと相違
//                this.WindowState = FormWindowState.Minimized;	//V3CSと相違
//                this.Hide();	//V3CSと相違
//            }
            SettingsLoad();
            Thread.CurrentThread.CurrentUICulture = new CultureInfo(FanduinaSettings.Culture);
            FicCreate();
            SidePanelCreate();
            SidePanel.Closed += MainFormClose;

            //タスクトレイアイコン設定
            if (FanduinaSettings.Culture == "ja-JP")
            {
                this.showSidePanelMenu.Text = "パネル再表示";
                this.controllerSettingsMenu.Text = "ファンコン設定";
                this.sidePanelConfiMenu.Text = "パネル表示設定";
                this.exitMenu.Text = "終　了";
            }
            ToolStripMenuItem item = new ToolStripMenuItem();
            for (Int32 i = 0; i < FanduinaSettings.FicMax; i++)
            {
                item = new ToolStripMenuItem(i.ToString());
                this.controllerSettingsMenu.DropDownItems.Add(item);
            }

            //起動時最小化
            if (FanduinaSettings.SidePanelStartMinimize == true)
            {
                SidePanel.WindowState = FormWindowState.Minimized;
            }
            else
            {
                SidePanel.WindowState = FormWindowState.Normal;
            }

            //代用温度計設定
            ExThermoClass.ExThermoEnumEventTimerAction(this,null);

        }
        //起動時にMainFormを非表示化	//V3CSと相違
		private void MainForm_Shown( object sender, EventArgs e )
		{
            this.WindowState = FormWindowState.Minimized;
            this.Hide();
		}
        //サイドパネルのインスタンスを起動
        private void SidePanelCreate()
        {
            SidePanel = new SidePanel();
			SidePanel.TopMost = FanduinaSettings.SidePanelAlwaysOnTop;	//V3CSと相違
            SidePanel.Show();
            if ( FanduinaSettings.SidePanelStartMinimize )
            {
                SidePanel.WindowState = FormWindowState.Minimized;	//V3CSと相違
				SidePanel.Visible = false;	//V3CSと相違
            }

        }
        //Ficのインスタンスを起動
        private void FicCreate()
        {
            for (Int32 i = 0; i < FanduinaSettings.FicMax; i++)
            {
                Fic[i] = new FanduinoInterface();
                Fic[i].Owner = this;
                Fic[i].Show();
                Fic[i].Hide();
                Fic[i].FicNum = i;
                Fic[i].PortName = FanduinaSettings.FicProp[i].PortName;
                Fic[i].PortConnect();
//                Fic[i].Opacity = 0.0;
//                Fic[i].Opacity = 1.0;
                Fic[i].AlarmFanEvent  += FicFan_Alarm;
                Fic[i].AlarmTempEvent += FicTemp_Alarm;
//				MessageBox.Show("Fic" + i.ToString() + " Create Complete");
            }
         }
        #endregion

        #region Settings 設定の保存・復元
        //設定の保存
        private void SettingsSave()
        {
            try
            {
                string fileName = exeDir + "/Settings/Settings.xml";	//V3CSと相違
                XmlSerializer serializer = new XmlSerializer(typeof(FanduinaSettings));
                FileStream fs = new FileStream(fileName, FileMode.Create);
                serializer.Serialize(fs, fanduinaSettings);
                fs.Close();
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrSettingsSave,Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK,MessageBoxIcon.Warning);
            }
        }
        //設定の復元
        private void SettingsLoad()
        {
            try
            {
                string fileName = exeDir + "/Settings/Settings.xml";	//V3CSと相違
                XmlSerializer serializer = new XmlSerializer(typeof(FanduinaSettings));
                FileStream fs = new FileStream(fileName, FileMode.Open);
                fanduinaSettings = (FanduinaSettings)serializer.Deserialize(fs);
                fs.Close();
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrSettingsLoad,Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK,MessageBoxIcon.Warning);
            }
        }
        #endregion

        #region Application End 終了処理
        private void MainFormClose(object sender, EventArgs e)
        {
            FanduinaV3Exit();
        }
        private void buttonExit_Click(object sender, EventArgs e)
        {
            FanduinaV3Exit();
        }
        private void FanduinaV3Exit()
        {
            SettingsSave();
//          this.Dispose();
            Environment.Exit(0);	//V3CSと相違
        }
        #endregion

        #region TaskTrayIcon タスクトレイアイコン
        //タスクトレイアイコン　右クリック時のメニュー
        private void showSidePanel_Click(object sender, EventArgs e)
        {
            SidePanel.WindowState = FormWindowState.Normal;
            SidePanel.Visible = true;	//V3CSと相違
			SidePanel.BringToFront();	//V3CSと相違
            //画面サイズを取得
            Int32 Display_Hight = Screen.GetBounds(this).Height;
            Int32 Display_Width = Screen.GetBounds(this).Width;
            //画面外の時は画面内に移動
            if (SidePanel.Left <= 0)
            {
                SidePanel.Left = 0;
            }
            else if (SidePanel.Right >= Display_Width)
            {
                SidePanel.Left = Display_Width - SidePanel.Size.Width;
            }
            if (SidePanel.Top <= 0)
            {
                SidePanel.Top = 0;
            }
            else if (SidePanel.Bottom >= Display_Hight)
            {
                SidePanel.Top = Display_Hight - SidePanel.Size.Height;
            }
        }
        private void sidePanelConfig_Click(object sender, EventArgs e)
        {
            SidePanel.sidePanelConfig_Click(sender, e);
        }
        private void controllerSettingMenu_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            Int32 i = Convert.ToInt32(e.ClickedItem.Text);
            Fic[i].Show();
        }
        private void exitFanduinaV3_Click(object sender, EventArgs e)
        {
            FanduinaV3Exit();
        }
        //タスクトレイアイコン　ダブルクリック時
        private void notifyIconFanduina_DoubleClick(object sender, EventArgs e)
        {
            showSidePanel_Click(sender, e);
        }
        #endregion

        #region Alarm アラーム処理
        private void FicFan_Alarm(object sender, AlarmFanEventArgs e)
        {
            if (e.AlarmByBaloon)
            {
                string FanBalntxt = "Controller" + e.FicNum.ToString() + " Fan" + (e.FanNum+1).ToString() + " Stopeed !";
                this.notifyIconFanduina.BalloonTipTitle = Properties.Resources.MsgFanStop;
                this.notifyIconFanduina.BalloonTipIcon = ToolTipIcon.Warning;
                this.notifyIconFanduina.BalloonTipText = FanBalntxt;
                this.notifyIconFanduina.ShowBalloonTip(10000); //10sec
                this.notifyIconFanduina.BalloonTipIcon = ToolTipIcon.None;
            }
            if (e.AlarmBySound)
            {
                System.Media.SoundPlayer player = new System.Media.SoundPlayer(exeDir + "/Settings/chime02.wav");	//V3CSと相違
                player.Play();
                player.Dispose();
            }
        }
        private void FicTemp_Alarm(object sender, AlarmTempEventArgs e)
        {
            if (e.AlarmByBaloon)
            {
                string TempBalntxt = "Controller" + e.FicNum.ToString() + " Temp" + (e.TempNum+1).ToString() + " OverHeat !";
                this.notifyIconFanduina.BalloonTipTitle = Properties.Resources.MsgOverTemp;
                this.notifyIconFanduina.BalloonTipIcon = ToolTipIcon.Warning;
                this.notifyIconFanduina.BalloonTipText = TempBalntxt;
                this.notifyIconFanduina.ShowBalloonTip(10000); //10sec
                this.notifyIconFanduina.BalloonTipIcon = ToolTipIcon.None;
            }
            if (e.AlarmBySound)
            {
                System.Media.SoundPlayer player = new System.Media.SoundPlayer(exeDir + "/Settings/chime01.wav");	//V3CSと相違
                player.Play();
                player.Dispose();
            }
        }
        #endregion

        #region ExThermo 代用温度計処理
        private void timerEnumTemps_Tick(object sender, EventArgs e)
        {
            ExThermoClass.ExThermoEnumEventTimerAction(this,null);
        }
        #endregion
    }

    #region ExthermoClass 代用温度計用のクラス
    static public class ExThermoClass          //情報を取得し列挙する
    {
        static private ExThermoEnumArgs OTHdata = new ExThermoEnumArgs(ExThermo.Others);
        static private ExThermoEnumArgs CPUdata = new ExThermoEnumArgs(ExThermo.CPU);
        static private ExThermoEnumArgs GEFdata = new ExThermoEnumArgs(ExThermo.GeForce);	//V3CSと相違
        static private ExThermoEnumArgs HDDdata = new ExThermoEnumArgs(ExThermo.HDD);
        static private ExThermoEnumArgs RADdata = new ExThermoEnumArgs(ExThermo.Radeon);	//V3CSと相違
        static private bool dataOTHUsed = false;
        static private bool dataCPUUsed = false;
        static private bool dataGEFUsed = false;	//V3CSと相違
        static private bool dataHDDUsed = false;
        static private bool dataRADUsed  = false;	//V3CSと相違

        static public void ExThermoEnumRequest(bool rapid, ref ExThermoEnumArgs e) //データ要求があったときは定期取得データを渡す
        {
            switch (e.ExThermoType)
            {
                case ExThermo.Others:
                    if (!OTHdata.Success || rapid) ExThermoEnum(OTHdata);
                    e = OTHdata;
                    dataOTHUsed = true;   //Extdataが利用されたことを記録
                    break;
                case ExThermo.CPU:
                    if (!CPUdata.Success || rapid) ExThermoEnum(CPUdata);
                    e = CPUdata;
                    dataCPUUsed = true;   //CPUdataが利用されたことを記録
                    break;
                case ExThermo.GeForce:	//V3CSと相違
                    if (!GEFdata.Success || rapid) ExThermoEnum(GEFdata);
                    e = GEFdata;
                    dataGEFUsed = true;   //GEFdataが利用されたことを記録
                    break;
                case ExThermo.HDD:
                    if (!HDDdata.Success || rapid) ExThermoEnum(HDDdata);
                    e = HDDdata;
                    dataHDDUsed = true;   //HDDdataが利用されたことを記録
                    break;
                case ExThermo.Radeon:	//V3CSと相違
                    if (!RADdata.Success || rapid) ExThermoEnum(RADdata);
                    e = RADdata;
                    dataRADUsed = true;   //RADdataが利用されたことを記録
                    break;
            }

        }
        static public void ExThermoEnumEventTimerAction(object sender, EventArgs e)   //タイマーによる定期的な温度データの取得
        {
            bool result;
            //External
            if (dataOTHUsed)   //Extデータが利用されていた時
            {
                OTHdata.ExThermoType = ExThermo.Others;
                result = ExThermoEnum(OTHdata);
                dataOTHUsed = false;
            }
            //CPU
            if (dataCPUUsed)   //CPUデータが利用されていた時
            {
                CPUdata.ExThermoType = ExThermo.CPU;
                result = ExThermoEnum(CPUdata);
                dataCPUUsed = false;
            }
            //GeForce
            if (dataGEFUsed)   //GeForceデータが利用されていた時	//V3CSと相違
            {
                GEFdata.ExThermoType = ExThermo.GeForce;
                result = ExThermoEnum(GEFdata);
                dataGEFUsed = false;
            }
            //HDD
            if (dataHDDUsed)   //HDDデータが利用されていた時
            {
                HDDdata.ExThermoType = ExThermo.HDD;
                result = ExThermoEnum(HDDdata);
                dataHDDUsed = false;
            }
            //Radeon
            if (dataRADUsed)   //Radeonデータが利用されていた時	//V3CSと相違
            {
                RADdata.ExThermoType = ExThermo.Radeon;
                result = ExThermoEnum(RADdata);
                dataRADUsed = false;
            }
        }
        static public bool ExThermoEnum(ExThermoEnumArgs e)                           //各種温度データを取得し登録
        {
            switch (e.ExThermoType)
            {
                case ExThermo.Others:
                    OTHEnum(e);
                    return e.Success;
                case ExThermo.CPU:
                    CPUEnum(e);
                    return e.Success;
                case ExThermo.GeForce:	//V3CSと相違
                    GeForceEnum(e);
                    return e.Success;
                case ExThermo.HDD:
                    HDDEnum(e);
                    return e.Success;
                case ExThermo.Radeon:	//V3CSと相違
                    RadeonEnum(e);
                    return e.Success;
                default: return false;
            }
        }
        #region Other Controller's Sensor 他のファンコンのセンサを利用
        static private void OTHEnum(ExThermoEnumArgs e)
        {
            int exThermoNum = 0;               //温度報告数
            for (int fic = 0; fic < FanduinaSettings.FicMax; fic++)
            {
                if (MainForm.Fic[fic].PortName != "off")
                {
                    for (int thm = 0; thm < MainForm.Fic[fic].TempCount; thm++)
                    {
                        e.ExThermoName[exThermoNum] = "Controller" + fic.ToString() + ":Temp" + (thm+1).ToString();
                        e.ExThermoValue[exThermoNum] = MainForm.Fic[fic].TempData[thm].TempValue;
                        exThermoNum++;
                    }
                }
            }
            e.ExThermoCount = exThermoNum;
            e.Success = true;
        }
        #endregion
        #region lm-sensors CPU情報の取得	//V3CSと相違
        static private void CPUEnum(ExThermoEnumArgs e)
        {
            e.Success = false;
    		// コマンドラインとしてlm-sensorsを実行
    		Process p = new Process();
    		p.StartInfo.FileName = "bash";				
			p.StartInfo.Arguments = FanduinaSettings.LmSensorsCommand;
    		p.StartInfo.CreateNoWindow = true;			// コンソール・ウィンドウを開かない
    		p.StartInfo.UseShellExecute = false;		// シェル機能を使用しない
    		p.StartInfo.RedirectStandardOutput = true;	// 標準出力をリダイレクト
			try
			{
				// 子プロセスの実行開始
				p.Start(); 									
				string output = p.StandardOutput.ReadToEnd();//出力を読み取る
    			p.WaitForExit(); // 子プロセスが終了するのを待つ
    			p.Dispose(); // 子プロセスの破棄
			    string[] stLines = new string[20];
            	//データの解析
                stLines = output.Trim().Replace("\r\n","\n").Split('\n');
				int exThermoNum= stLines.Length;
				for (int i = 0; i < exThermoNum; i++)
				{
					string[] stSplit = new string[10];
					stSplit = stLines[i].Split(':');
					e.ExThermoName[i]  = stSplit[0];
					e.ExThermoValue[i] = Convert.ToDouble(stSplit[1].Substring(0,stSplit[1].IndexOf("°C")));
				}
           		e.ExThermoCount = exThermoNum;	//温度報告数
           		e.Success = true;
            }
            catch
			{
                MessageBox.Show(Properties.Resources.ErrLmSensorsRun, Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
        }
        #endregion
        #region HDD Temp HDD情報の取得	//V3CSと相違
        static private void HDDEnum(ExThermoEnumArgs e)
        {
            e.Success = false;
    		// コマンドラインとしてhddtempを実行
    		Process p = new Process();
    		p.StartInfo.FileName = "bash";				
			p.StartInfo.Arguments = FanduinaSettings.HddTempCommand;
    		p.StartInfo.CreateNoWindow = false;			// コンソール・ウィンドウを開かない
    		p.StartInfo.UseShellExecute = false;		// シェル機能を使用しない
    		p.StartInfo.RedirectStandardOutput = true;	// 標準出力をリダイレクト
			try
			{
				// 子プロセスの実行開始
				p.Start(); 									
				string output = p.StandardOutput.ReadToEnd();//出力を読み取る
    			p.WaitForExit(); // 子プロセスが終了するのを待つ
    			p.Dispose(); // 子プロセスの破棄
			    string[] stLines = new string[20];
            	//データの解析
                stLines = output.Trim().Replace("\r\n","\n").Split('\n');
				int exThermoNum= stLines.Length;
				for (int i = 0; i < exThermoNum; i++)
				{
					string[] stSplit = new string[10];
					stSplit = stLines[i].Split(':');
					e.ExThermoName[i]  = stSplit[1];
					e.ExThermoValue[i] = Convert.ToDouble(stSplit[2].Substring(0,stSplit[2].IndexOf("°C")));
				}
           		e.ExThermoCount = exThermoNum;	//温度報告数
           		e.Success = true;
            }
            catch
			{
                MessageBox.Show(Properties.Resources.ErrHddTempRun, Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
        }
        #endregion
        #region GeForce GPU温度情報の取得	//V3CSと相違
        static private void GeForceEnum(ExThermoEnumArgs e)
        {
            e.Success = false;
    		// コマンドラインとしてnvidia-settingsを実行
    		Process p = new Process();
    		p.StartInfo.FileName = "bash";				
			p.StartInfo.Arguments = FanduinaSettings.GeForceCommand;
    		p.StartInfo.CreateNoWindow = false;			// コンソール・ウィンドウを開かない
    		p.StartInfo.UseShellExecute = false;		// シェル機能を使用しない
    		p.StartInfo.RedirectStandardOutput = true;	// 標準出力をリダイレクト
			try
			{
				// 子プロセスの実行開始
				p.Start(); 									
				string output = p.StandardOutput.ReadToEnd();//出力を読み取る
    			p.WaitForExit(); // 子プロセスが終了するのを待つ
    			p.Dispose(); // 子プロセスの破棄
			    string[] stLines = new string[10];
            	//データの解析
                stLines = output.Trim().Replace("\r\n","\n").Split('\n');
				int exThermoNum= stLines.Length;
				for (int i = 0; i < exThermoNum; i++)
				{
//					string[] stSplit = new string[10];
//					stSplit = stLines[i].Split('-');
					e.ExThermoName[i]  = "GPU Core" + i.ToString();
					e.ExThermoValue[i] = Convert.ToDouble(stLines[i]);
				}
           		e.ExThermoCount = exThermoNum;	//温度報告数
           		e.Success = true;
            }
            catch
			{
                MessageBox.Show(Properties.Resources.ErrGeForceRun, Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
        }
        #endregion
       #region Radeon GPU温度情報の取得	//V3CSと相違
        static private void RadeonEnum(ExThermoEnumArgs e)
        {
            e.Success = false;
    		// コマンドラインとしてaticonfigを実行
    		Process p = new Process();
    		p.StartInfo.FileName = "bash";				
			p.StartInfo.Arguments = FanduinaSettings.RadeonCommand;
    		p.StartInfo.CreateNoWindow = false;			// コンソール・ウィンドウを開かない
    		p.StartInfo.UseShellExecute = false;		// シェル機能を使用しない
    		p.StartInfo.RedirectStandardOutput = true;	// 標準出力をリダイレクト
			try
			{
				// 子プロセスの実行開始
				p.Start(); 									
				string output = p.StandardOutput.ReadToEnd();//出力を読み取る
    			p.WaitForExit(); // 子プロセスが終了するのを待つ
    			p.Dispose(); // 子プロセスの破棄
			    string[] stLines = new string[10];
            	//データの解析
                stLines = output.Trim().Replace("\r\n","\n").Split('\n');
				int exThermoNum= stLines.Length;
				for (int i = 0; i < exThermoNum; i++)
				{
					string[] stSplit = new string[10];
					stSplit = stLines[i].Split('-');
					e.ExThermoName[i]  = stSplit[0];
					e.ExThermoValue[i] = Convert.ToDouble(stSplit[1].Substring(0,stSplit[1].IndexOf("C")));
				}
           		e.ExThermoCount = exThermoNum;	//温度報告数
           		e.Success = true;
            }
            catch
			{
                MessageBox.Show(Properties.Resources.ErrRadeonRun, Properties.Resources.WarningMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
			}
        }
        #endregion
    }
    public class ExThermoEnumArgs       //代用温度計のリストと現在値を要求する引数用のクラス
    {
        public bool Success { get; set; }
        public Int32 FicNum { get; set; }
        public Int32 TempNum { get; set; }
        public Int32 ExThermoType { get; set; }
        public Int32 ExThermoCount { get; set; }
        public String[] ExThermoName { get; set; }
        public Double[] ExThermoValue { get; set; }
        public ExThermoEnumArgs(Int32 val)
        {
            Success = false;
            FicNum = 0;
            TempNum = 0;
            ExThermoType = val;
            ExThermoCount = 0;
            ExThermoName = new String[20];
            ExThermoValue = new Double[20];
        }
    }
    #endregion

    #region Settings Class 設定値の保存クラス
    [Serializable()]
    public class FanduinaSettings          
    {
        //静的フィールド
        static public string   Culture;                            //表示する言語
        static public Int32    FicMax;                             //ファンコンの接続数（最大４まで）
        static public Int32    FicFanMax;                          //ファンコン１台の最大ファン接続数
        static public Int32    FicTempMax;                         //ファンコン１台の最大温度計接続数
        static public Point    SidePanelLocation;                  //起動時表示位置
        static public bool     SidePanelAlwaysOnTop;               //最前面化
        static public bool     SidePanelStartMinimize;             //起動時最小化
        static public bool     SidePanelSnapToEdge;                //画面端吸着
        static public bool     SidePanelMouseOverInvisible;        //マウスオーバー時非表示
        static public double   SidePanelMouseOverInvisibleTimer;   //非表示からの復帰時間(sec)
        static public double   SidePanelOpacity;                   //不透明度
        static public double   SidePanelResizeRate;                //拡大縮小率
        static public string   SidePanelFontName;                  //フォント名称
        static public Int32    SidePanelFontSize;                  //フォントサイズ
        static public Int32    FansFanCount;                       //ファン表示数
        static public Int32    FansTitleColorInt;                  //ファンタイトル表示色
        static public Int32    FansBackColorInt;                   //ファン背景色
        static public Int32    FansBarColorInt;                    //ファングラフ表示色
        static public bool     FansSameAsTitleColor;               //ファン表示色をタイトル色に
        static public bool     FansSameAsTempColor;                //ファン表示色を温度色に
        static public Int32    TempsTempCount;                     //温度表示数
        static public Int32    TempsTitleColorInt;                 //温度タイトル表示色
        static public Int32    TempsBackColorInt;                  //温度背景色
        static public Int32    TempsBarColorInt;					//温度グラフ表示色
        static public bool     TempsSameAsTitleColor;              //温度表示色をタイトル色に
        static public string   LmSensorsCommand;                   //lm-sensorsの起動コマンド	//V3CSと相違
        static public string   HddTempCommand;                     //hddtempの起動コマンド	//V3CSと相違
		static public string   GeForceCommand;    				   //nvidia-settingsの起動コマンド	//V3CSと相違
        static public string   RadeonCommand;                      //aticonfigの起動コマンド	//V3CSと相違
        static public string   ReadmeCommand;                      //ReadMe.txtを読むエディタの起動コマンド	//V3CSと相違
        static public FanDispPropStruct[]  FanDispProp;            //個別ファン表示の設定
        static public TempDispPropStruct[] TempDispProp;           //個別温度表示の設定
        static public FicPropStruct[]      FicProp;                //個別ファンコンの設定
        //プロパティ
        public string   culture                            //表示する言語
        {
            get { return Culture; }
            set { Culture = value; }
        }
        public Int32    ficMax                             //ファンコンの接続数（最大４まで）
        {
            get { return FicMax; }
            set { FicMax = value; }
        }
        public Int32    fcicFanMax                          //ファンコン１台の最大ファン接続数
        {
            get { return FicFanMax; }
            set { FicFanMax = value; }
        }
        public Int32    ficTempMax                         //ファンコン１台の最大温度計接続数
        {
            get { return FicTempMax; }
            set { FicTempMax = value; }
        }
        public Point    sidePanelLocation                  //起動時表示位置
        {
            get { return SidePanelLocation; }
            set { SidePanelLocation = value; }
        }
        public bool     sidePanelAlwaysOnTop               //最前面化
        {
            get { return SidePanelAlwaysOnTop; }
            set { SidePanelAlwaysOnTop = value; }
        }
        public bool     sidePanelStartMinimize             //起動時最小化
        {
            get { return SidePanelStartMinimize; }
            set { SidePanelStartMinimize = value; }
        }
        public bool     sidePanelSnapToEdge                //画面端吸着
        {
            get { return SidePanelSnapToEdge; }
            set { SidePanelSnapToEdge = value; }
        }
        public bool     sidePanelMouseOverInvisible        //マウスオーバー時非表示
        {
            get { return SidePanelMouseOverInvisible; }
            set { SidePanelMouseOverInvisible = value; }
        }
        public double   sidePanelMouseOverInvisibleTimer   //非表示からの復帰時間(sec)
        {
            get { return SidePanelMouseOverInvisibleTimer; }
            set { SidePanelMouseOverInvisibleTimer = value; }
        }
        public double   sidePanelOpacity                   //不透明度
        {
            get { return SidePanelOpacity; }
            set { SidePanelOpacity = value; }
        }
        public double   sidePanelResizeRate                //拡大縮小率
        {
            get { return SidePanelResizeRate; }
            set { SidePanelResizeRate = value; }
        }
        public string   sidePanelFontName                  //フォント名称
        {
            get { return SidePanelFontName; }
            set { SidePanelFontName = value; }
        }
        public Int32    sidePanelFontSize                  //フォントサイズ
        {
            get { return SidePanelFontSize; }
            set { SidePanelFontSize = value; }
        }
        public Int32    fansFanCount                       //ファン表示数
        {
            get { return FansFanCount; }
            set { FansFanCount = value; }
        }
        public Int32    fansTitleColorInt                  //ファンタイトル表示色
        {
            get { return FansTitleColorInt; }
            set { FansTitleColorInt = value; }
        }
        public Int32    fansBackColorInt                   //ファン背景色
        {
            get { return FansBackColorInt; }
            set { FansBackColorInt = value; }
        }
        public Int32    fansBarColorInt                    //ファングラフ表示色
        {
            get { return FansBarColorInt; }
            set { FansBarColorInt = value; }
        }
        public bool     fansSameAsTitleColor               //ファン表示色をタイトル色に
        {
            get { return FansSameAsTitleColor; }
            set { FansSameAsTitleColor = value; }
        }
        public bool     fansSameAsTempColor                //ファン表示色を温度色に
        {
            get { return FansSameAsTempColor; }
            set { FansSameAsTempColor = value; }
        }
        public Int32    tempsTempCount                     //温度表示数
        {
            get { return TempsTempCount; }
            set { TempsTempCount = value; }
        }
        public Int32    tempsTitleColorInt                 //温度タイトル表示色
        {
            get { return TempsTitleColorInt; }
            set { TempsTitleColorInt = value; }
        }
        public Int32    tempsBackColorInt                  //温度背景色
        {
            get { return TempsBackColorInt; }
            set { TempsBackColorInt = value; }
        }
        public Int32    tempsBarColorInt					//温度グラフ表示色
        {
            get { return TempsBarColorInt; }
            set { TempsBarColorInt = value; }
        }
        public bool     tempsSameAsTitleColor              //温度表示色をタイトル色に
        {
            get { return TempsSameAsTitleColor; }
            set { TempsSameAsTitleColor = value; }
        }
        public string   lmSensorsCommand                   //lm-sensorsの起動コマンド	//V3CSと相違
        {
            get { return LmSensorsCommand; }
            set { LmSensorsCommand = value; }
        }
        public string   hddTempCommand                     //hddtempの起動コマンド	//V3CSと相違
        {
            get { return HddTempCommand; }
            set { HddTempCommand = value; }
        }
        public string   geForceCommand                     //nvidia-settingsの起動コマンド	//V3CSと相違
        {
            get { return GeForceCommand; }
            set { GeForceCommand = value; }
        }
        public string   radeonCommand                     //aticonfigの起動コマンド	//V3CSと相違
        {
            get { return RadeonCommand; }
            set { RadeonCommand = value; }
        }
        public string   readmeCommand                     //Readme.txtを読むエディタの起動コマンド	//V3CSと相違
        {
            get { return ReadmeCommand; }
            set { ReadmeCommand = value; }
        }
        public FanDispPropStruct[]  fanDispProp            //個別ファン表示の設定
        {
            get { return FanDispProp; }
            set { FanDispProp = value; }
        }
        public TempDispPropStruct[] tempDispProp           //個別温度表示の設定
        {
            get { return TempDispProp; }
            set { TempDispProp = value; }
        }
        public FicPropStruct[]      ficProp                //個別ファンコンの設定
        {
            get { return FicProp; }
            set { FicProp = value; }
        }
        //コンストラクタ（デフォルト値に設定）
        public FanduinaSettings()
        {
            //Culture = CultureInfo.CurrentCulture.ToString();
            Culture = "ja-JP";
            FicMax     = 4;
            FicFanMax  = 6;
            FicTempMax = 4;
            SidePanelLocation = new Point(0, 0);
            SidePanelAlwaysOnTop = false;
            SidePanelStartMinimize = false;
            SidePanelSnapToEdge = false;
            SidePanelMouseOverInvisible = false;
            SidePanelMouseOverInvisibleTimer = 0.1;
            SidePanelOpacity = 1.0;
            SidePanelResizeRate = 1.0;
            SidePanelFontName = "tahoma";
            SidePanelFontSize = 9;
            FansFanCount = 6;
            FansTitleColorInt = Color.White.ToArgb();
            FansBackColorInt = Color.Black.ToArgb();
            FansBarColorInt = Color.Blue.ToArgb();
            FansSameAsTitleColor = true;
            FansSameAsTempColor = false;
            TempsTempCount = 4;
            TempsTitleColorInt = Color.White.ToArgb();
            TempsBackColorInt = Color.Black.ToArgb();
            TempsBarColorInt = Color.Red.ToArgb();
            TempsSameAsTitleColor = true;
            LmSensorsCommand = "-c 'sensors | grep -e '°C''";	//V3CSと相違
            HddTempCommand   = "-c 'hddtemp /dev/sda | grep -e '°C''";	//V3CSと相違
			GeForceCommand   = "-c 'nvidia-settings -tq GPUCoreTemp'";	//V3CSと相違
			RadeonCommand    = "-c 'aticonfig --odgt | grep -e 'Temperature''";	//V3CSと相違
			ReadmeCommand    = "/usr/bin/gedit ";	//V3CSと相違
            FanDispProp = new FanDispPropStruct[FicMax * FicFanMax];
            TempDispProp = new TempDispPropStruct[FicMax * FicTempMax];
            FicProp = new FicPropStruct[FicMax];
            for (int i = 0; i < FicMax; i++)
            {
                for (int j = 0; j < FicFanMax; j++)
                {
                    FanDispProp[i * FicFanMax + j] = new FanDispPropStruct(i, j);
                }
                for (int j = 0; j < FicTempMax; j++)
                {
                    TempDispProp[i * FicTempMax + j] = new TempDispPropStruct(i, j);
                }
                FicProp[i] = new FicPropStruct(i);
            }
        }
    }
    #endregion

    #region StructDeclare 各種データの構造体
    public struct FanDataStruct         //個別ファンデータの構造体
    {

        public Int32  FicNum;       //ファンコン番号（０～）
        public Int32  FanNum;       //ファン番号(０～）
        public string FanName;
        public Int32  FanTach;
        public double FanDuty;
        public FanDataStruct(Int32 val1, Int32 val2)
        {
            this.FicNum  = val1;
            this.FanNum  = val2;
            this.FanName = "Fan";
            this.FanTach = 0;
            this.FanDuty = 100;
        }
    }
    public struct TempDataStruct        //個別温度データの構造体
    {
        public Int32  FicNum;       //ファンコン番号（０～）
        public Int32  TempNum;      //温度計番号(０～）
        public string TempName;
        public double TempValue;
        public TempDataStruct(Int32 val1, Int32 val2)
        {
            this.FicNum    = val1;
            this.TempNum   = val2;
            this.TempName  = "Temp";
            this.TempValue = 30.0;
        }
    }
    public struct FanDispPropStruct     //ファンパネルの個別ファン表示用プロパティ構造体
    {
        public Int32 FicNum;            //ファンコン番号
        public Int32 FanNum;            //ファン番号
        public Int32 FanTextColorInt;   //文字表示色
        public Int32 FanBarColorInt;    //グラフ表示色
        public FanDispPropStruct(Int32 val1, Int32 val2)
        {
            this.FicNum = val1;
            this.FanNum = val2;
            this.FanTextColorInt = Color.Black.ToArgb();
            this.FanBarColorInt  = Color.Blue.ToArgb();
        }
    }
    public struct TempDispPropStruct    //温度パネルの個別温度表示用プロパティ構造体
    {
        public Int32 FicNum;            //ファンコン番号
        public Int32 TempNum;           //温度番号
        public Int32 TempTextColorInt;  //文字表示色
        public Int32 TempBarColorInt;   //グラフ表示色
        public TempDispPropStruct(Int32 val1, Int32 val2)
        {
            this.FicNum  = val1;
            this.TempNum = val2;
            this.TempTextColorInt = Color.Black.ToArgb();
            this.TempBarColorInt  = Color.Red.ToArgb();
        }
    }
    public struct FicPropStruct         //ファンコン毎の設定
    {
        public Int32    FicNum;              //ファンコン番号
        public Int32    FanCount;            //ファンの数
        public Int32    TempCount;           //温度計の数
        public Int32    PwmCount;            //ファン制御用PWMの数
        public string   PortName;            //ポート名
        public bool     LogEnable;           //ログ記録の有無
        public Int32    LogInterval;         //ログ間隔（秒）
        public Int32    AlarmTempTimer;      //温度アラームまでの時間（秒）
        public bool     AlarmByBalloon;      //バルーンでアラーム
        public bool     AlarmBySound;        //音でアラーム
        public bool[]   AlarmTempEnable;     //温度アラームの有無
        public Int32[]  AlarmTempValue;      //アラーム設定温度
        public bool[]   AlarmFanEnable;      //ファン停止アラームの有無
        public Int32[]  ExThermoType;        //代用温度計のタイプ
        public string[] ExThermoName;        //代用温度計の名前
        public Int32[]  ExThermoAdjust;      //代用温度計の補正値
        public FicPropStruct(Int32 val1)
        {
            this.FicNum         = val1;                //入力値
            this.FanCount       =   6;                 //以下デフォルト値を設定
            this.TempCount      =   4;
            this.PwmCount       =   4;
            this.PortName       = "off";
            this.LogEnable      = false;
            this.LogInterval    =  60;
            this.AlarmTempTimer =  30;
            this.AlarmByBalloon = false;
            this.AlarmBySound   = false;
            AlarmTempEnable = new bool[4];
            AlarmTempValue  = new Int32[4];
            AlarmFanEnable  = new bool[6];
            ExThermoType    = new Int32[4];
            ExThermoName    = new string[4];
            ExThermoAdjust  = new Int32[4];
            for (int i = 0; i < TempCount; i++)
            {
                this.AlarmTempEnable[i] = false;
                this.AlarmTempValue[i]  = 100;
                this.ExThermoType[i]    = 0;
                this.ExThermoName[i]    = " ";	//V3CSと相違
                this.ExThermoAdjust[i]  = 0;
            }
            for (int i = 0; i < FanCount; i++)
            {
                this.AlarmFanEnable[i] = false;
            }
        }
    }
    public struct FanParamStruct        //ファン制御パラメータの構造体
    {
        public Int32  FicNum;        //ファンコン番号（０～）
        public Int32  FanNum;        //ファン番号(０～）
        public string FanName;       //ファン名称
        public Int32  Tmax;
        public Int32  Tmin;
        public Int32  Tstop;
        public Int32  Dmax;
        public Int32  Dmin;
        public Int32  Dman;
        public Int32  FanThm;
        public Int32  FanMan;
        public FanParamStruct(Int32 val1, Int32 val2)
        {
            this.FicNum  = val1;
            this.FanNum  = val2;
            this.FanName = "Fan";
            this.Tmax    =  60;
            this.Tmin    =  30;
            this.Tstop   =  25;
            this.Dmax    = 100;
            this.Dmin    =  30;
            this.Dman    = 100;
            this.FanThm  =   1;
            this.FanMan  =   0; //自動
        }
    }
    public struct TempParamStruct       //温度パラメータの構造体
    {
        public Int32  FicNum;       //ファンコン番号（０～）
        public Int32  TempNum;      //温度計番号(０～）
        public string TempName;     //温度計名称
        public TempParamStruct(Int32 val1, Int32 val2)
        {
            this.FicNum   = val1;
            this.TempNum  = val2;
            this.TempName = "Temp";
        }
    }
    #endregion

    #region Global Constants 定数
    abstract class ExThermo             //代用温度計の定数
    {
        public const Int32 OwnSensor = 0;
        public const Int32 Others    = 1;
        public const Int32 CPU       = 2;
        public const Int32 GeForce   = 3;	//V3CSと相違
        public const Int32 HDD       = 4;
        public const Int32 Radeon    = 5;	//V3CSと相違
    }
    #endregion
}
