﻿using System;
using System.Collections.Generic;
using System.IO.Ports;	//V3CSと相違
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;	//V3CSと相違
using System.Windows.Forms;
using MonoLibUsb;	//V3CSと相違
using MonoLibUsb.Descriptors;	//V3CSと相違
using MonoLibUsb.Profile;	//V3CSと相違
using MonoLibUsb.Transfer;	//V3CSと相違
using LibUsbDotNet;	//V3CSと相違
using LibUsbDotNet.Main;	//V3CSと相違

namespace FanduinaV3CS
{
    //----------------------------------------------------------------------------------------------------
    // USBタイプのシリアルポートを抜くとアプリケーションが異常終了する現象を回避するための SerialPort 継承クラス   
    // http://ivis-mynikki.blogspot.com/2009/10/c-serialportunauthorizedaccessexception.html  を参照  
    public class SafeSerialPort : System.IO.Ports.SerialPort   
    {   
        bool userOpen = false;   
        System.IO.Stream SafeBaseStream = null;   
 
        public new void Open()   
        {   
            base.Open();   
            userOpen = true;   
            SafeBaseStream = this.BaseStream;
            GC.SuppressFinalize(SafeBaseStream);    //追加
        }  
 
        protected override void Dispose(bool disposing)   
        {
            if (userOpen && base.IsOpen == false)
            {   
                try  
                {
                    userOpen = false;   
                    if (disposing)   
                    {   
                        SafeBaseStream.Close();
                        GC.ReRegisterForFinalize(SafeBaseStream);   //追加
                    }   
                }   
                catch(UnauthorizedAccessException)   
                {   
                }   
            }   
            base.Dispose(disposing);   
        }   
    }  

    //----------------------------------------------------------------------------------------------------
    // USB HIDデバイス用のクラス
    // HIDデバイスを列挙する
    public class HidEnumClass	//V3CSと相違
    {
        public Int32   		ModelMax;                           //VenderID,ProductID,HidTypeをセットとしたHIDデバイスの種類の数
		public UInt16[] 	VendorID  = new UInt16[10];         //ベンダーID
        public UInt16[]  	ProductID = new UInt16[10];         //製品ID
        public Int32[]   	HidType   = new Int32[10];          //HID転送のタイプ（コントロール転送、インタラプト転送）
		public Int32[]		RepLength = new Int32[10];			//HID転送のレコード長
		public Int32     	NumberOfDevs;                       //検出されたHIDデバイスの数
        public byte[]   	VersionNO = new byte[16];           //HIDデバイスのバージョンNO.（ファーム番号）
        public byte[]   	SerialNO  = new byte[16];           //HIDデバイスのシリアルNO.（個体識別用番号）
        public string[] 	HidName   = new string[16];         //HIDデバイスのポート番号（HID0,HID1,・・・）
        public Int32[]   	HidModel  = new Int32[16];          //HIDデバイスのモデル番号（0:Rev8.0, 1:Rev6.0,・・・）
        public MonoUsbProfile[] 	Profile = new MonoUsbProfile[16];    //HIDデバイスのプロファイル

		private MonoUsbSessionHandle SessionHandle;

        public HidEnumClass()                               //HidEnumClassのコンストラクタ
        {
        }
        public bool EnumHidDevs()                           //USB HIDデバイスを列挙するメソッド（エラー時はFalseを返す）
        {
			SessionHandle = new MonoUsbSessionHandle();
			MonoUsbProfileList profileList = new MonoUsbProfileList();
			int ret = profileList.Refresh(SessionHandle);
			if (ret < 0) return false;
			NumberOfDevs = 0;
			bool MyDeviceDetected = false;
			foreach ( MonoUsbProfile profile in profileList )
			{
            	for (int Model = 0; Model < ModelMax; Model++)
				{
					if (profile.DeviceDescriptor.VendorID == VendorID[Model] && 
					    profile.DeviceDescriptor.ProductID == ProductID[Model])
					{
						MyDeviceDetected = true;
						VersionNO[NumberOfDevs] = Convert.ToByte((profile.DeviceDescriptor.BcdDevice >> 8) & 0xff);
                        SerialNO[NumberOfDevs]  = Convert.ToByte(profile.DeviceDescriptor.BcdDevice & 0xff);
						Profile[NumberOfDevs]  = profile;
                        HidName[NumberOfDevs]   = "HID" + SerialNO[NumberOfDevs].ToString();
                        HidModel[NumberOfDevs]  = Model;
//						MessageBox.Show( "Found HID device \r\n" + 
//						                 " VendorID: "  + Convert.ToString(profile.DeviceDescriptor.VendorID,16) +
//						                 " ProductID: " + Convert.ToString(profile.DeviceDescriptor.ProductID,16) + "\r\n" +
//						                 " VersionNo.: " + VersionNO[NumberOfDevs].ToString() +
//						                 " SerialNo. : " + SerialNO[NumberOfDevs].ToString() + "\r\n" +
//										 " ProfileHandle: " + ProfileHandle[NumberOfDevs].ToString() + "\r\n" +
//										 " SessionHandle: " + SessionHandle + "\r\n" +
//						                 " HidName : " + HidName[NumberOfDevs] );
                        NumberOfDevs += 1;
					}
				}
			}
			profileList.Close();
			SessionHandle.Close();
			return MyDeviceDetected;
		}
    }

    // HIDデバイスと通信する
    public class HidStringClass	//V3CSと相違
    {
        public UInt16   	VendorID;               //HIDデバイスのベンダーID
        public UInt16   	ProductID;              //HIDデバイスの製品ID
        public Int32    	HidType;                //HID転送のタイプ（コントロール転送、インタラプト転送）
        public byte     	VersionNO;              //HIDデバイスのバージョンNO.（ファーム番号）
        public byte     	SerialNO;               //HIDデバイスのシリアルNO.（個体識別用番号）
        public IntPtr   	HandleID;               //HIDデバイスにアクセスするためのハンドラ
        public Int32    	ReportLength;           //送受信時の1行のレコード長（ヘッダ1バイト分を含む）
        public bool     	IsOpen;                 //HIDデバイスがオープンされている時trueを返す
        public MonoUsbProfile	Profile;    		//HIDデバイスのプロファイル
		
		private MonoUsbSessionHandle SessionHandle;
		private MonoUsbDeviceHandle deviceHandle = null;
		private int   InterfaceNumber = 0;
        private const byte  requestTypeRead  = (byte)(UsbCtrlFlags.Direction_In  | UsbCtrlFlags.Recipient_Interface | UsbCtrlFlags.RequestType_Class);
        private const byte  requestTypeWrite = (byte)(UsbCtrlFlags.Direction_Out | UsbCtrlFlags.Recipient_Interface | UsbCtrlFlags.RequestType_Class);
        private const byte  requestRead  = 1;
		private const byte  requestWrite = 9;
        private const short valueRead    = 3<<8;
        private const short valueWrite   = 3<<8;

        public          HidStringClass()        //HisStringClassのコンストラクタ
        {
            IsOpen = false;
        }
        public bool     Open()                  //HIDデバイスのオープンメソッド
        {
			SessionHandle = new MonoUsbSessionHandle();
			MonoUsbProfileList profileList = new MonoUsbProfileList();
			int ret = profileList.Refresh(SessionHandle);
			if (ret < 0) return false;
			bool MyDeviceDetected = false;
			foreach ( MonoUsbProfile profile in profileList )
			{
 				if (profile.DeviceDescriptor.VendorID == VendorID && 
					profile.DeviceDescriptor.ProductID == ProductID &&
				    Convert.ToByte((profile.DeviceDescriptor.BcdDevice >> 8) & 0xff) == VersionNO &&
					Convert.ToByte(profile.DeviceDescriptor.BcdDevice & 0xff) == SerialNO)
				{
					MyDeviceDetected = true;
					try
					{
						deviceHandle = profile.OpenDeviceHandle();

                		MonoUsbError e;
						// Detach kernel driver
						e = (MonoUsbError) (ret = MonoUsbApi.DetachKernelDriver(deviceHandle,0));
//                		if (ret < 0) throw new Exception(String.Format("Failed DetachKernelDriver.\n{0}:{1}", e, MonoUsbApi.StrError(e)));
                		// Set Configuration
                		e = (MonoUsbError) (ret = MonoUsbApi.SetConfiguration(deviceHandle, 1));
                		if (ret < 0) throw new Exception(String.Format("Failed SetConfiguration.\n{0}:{1}", e, MonoUsbApi.StrError(e)));

                		// Claim Interface
                		e = (MonoUsbError) (ret = MonoUsbApi.ClaimInterface(deviceHandle, (int)InterfaceNumber));
                		if (ret < 0) throw new Exception(String.Format("Failed ClaimInterface.\n{0}:{1}", e, MonoUsbApi.StrError(e)));
						
					}
					catch(Exception ex)
					{
						MessageBox.Show(ex.ToString());
						return false;
					}
				}
			}
			profileList.Close();
			this.IsOpen = true;
//			MessageBox.Show( "ReportLength:" + ReportLength.ToString() );
			return MyDeviceDetected;
		}
	
        public bool     Close()                 //HIDデバイスのクローズメソッド
        {
            try
            {
				int ret = MonoUsbApi.AttachKernelDriver(deviceHandle,0);
				this.IsOpen = false;
				deviceHandle.Close();
				SessionHandle.Close();
            	return true;
            }
            catch
            {
				return false;
			}
        }
		
        public string   ReadString()            //HIDデバイスから１行分を読込みStringとして返すメソッド
        {
            byte[]  inputReportBuffer = new byte[65];
			string  retString = "";
            int     numOfBytes = 0;

            if (HidType == HID_Interrupt_Transfer)
            {
				int ret = MonoUsbApi.InterruptTransfer(deviceHandle, 0x81, inputReportBuffer,
				                                          64, out numOfBytes, 50);
				if (ret<0) return "";
			}
            else if (HidType == HID_Control_Transfer)
            {
                numOfBytes = MonoUsbApi.ControlTransfer(deviceHandle,requestTypeRead, requestRead, valueRead,
				                                    0, inputReportBuffer, 32, 100);
            }

            if (numOfBytes > 0)
            {
				int validBytes;
				for ( validBytes = 0; validBytes < numOfBytes; validBytes++ )
				{
					if ( inputReportBuffer[validBytes] == 0 ) break;
				}
				if ( validBytes > 0 )
				{
                	retString = System.Text.Encoding.UTF8.GetString(inputReportBuffer, 0, validBytes);
					retString = retString.Trim();
					return retString;
				}
            }
            return "";
        }
        public bool     WriteString(string st)  //HIDデバイスに１行(st)を書込むメソッド
	    {
//		    string  strData = null;
		    byte[]  outputReportBuffer = new byte[100];
		    byte[]  buff = new byte[100];
		    Int32   i = 0;
		    Int32   numOfBytes = 0;
//
//		    strData = st.PadRight(ReportLength,' ');
		    buff = System.Text.Encoding.UTF8.GetBytes(st);

			for (i = 0; i < st.Length; i++)
            {
			    outputReportBuffer[i] = buff[i];
		    }
		    if (st.Length < ReportLength) 
            {
			    for (i = st.Length; i < ReportLength; i++) 
                {
				    outputReportBuffer[i] = 0;
			    }
		    }

			try
            {
			    if (HidType == HID_Interrupt_Transfer)
                {
					int ret = MonoUsbApi.InterruptTransfer(deviceHandle, 0x02, outputReportBuffer,
				                                          64, out numOfBytes, 50);
					if (ret != 0) return false;
			    } 
                else if (HidType == HID_Control_Transfer) 
                {
                	numOfBytes = MonoUsbApi.ControlTransfer(deviceHandle,requestTypeWrite, requestWrite, valueWrite,
				                                    0, outputReportBuffer, 32, 100);
					if (numOfBytes == 0)
					{
//						MessageBox.Show("Error:HID");
						return false;
					}
			    }
		    } 
            catch
            {
		        return false;
		    }
		    return true;
	    }

        private const Int32 HID_Control_Transfer = 0;
        private const Int32 HID_Interrupt_Transfer = 1;
    }

}
