﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Diagnostics;	//V3CSと相違

namespace FanduinaV3CS
{

    public partial class SidePanel : Form
    {
        #region "ClassDef"
        //コンストラクタ
        public       SidePanel() 
        {
			SetStyle(ControlStyles.SupportsTransparentBackColor, true);	//背景に透明色を使えるように設定	//V3CSと相違
			InitializeComponent();
            Load += SidePanel_Load;
        }
        //ロード時初期化
        private void SidePanel_Load(object sender, EventArgs e) //サイドパネルをロードする
        {
            //SidePanelの初期設定
			exeDir = Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location).ToString();	//V3CSと相違
//			MessageBox.Show("SidePanel: " + exeDir );
            FileStream fs = File.OpenRead(exeDir + "/Settings/Back.gif");	//V3CSと相違
            Bitmap bmp1 = (Bitmap)Image.FromStream(fs, false, false);
            Color transColor = bmp1.GetPixel(0, 0);
            bmp1.MakeTransparent(transColor);
            sidePanelFrame.Image = bmp1;
            sidePanelFrame.BackColor = Color.Transparent;
            this.BackColor = Color.Transparent;
            Bitmap bmp2 = new Bitmap(exeDir + "/Settings/Logo.bmp");	//V3CSと相違
            Color transColor2 = bmp2.GetPixel(0,0);
            bmp2.MakeTransparent(transColor2);
            Logo_FanduinaV3.Image = bmp2;
            Logo_FanduinaV3.Parent = sidePanelFrame;
            Logo_FanduinaV3.BackColor = Color.Transparent;
            this.Location = FanduinaSettings.SidePanelLocation;
            ToolStripMenuItem item = new ToolStripMenuItem();
            for (Int32 i = 0; i < FanduinaSettings.FicMax; i++)
            {
                item = new ToolStripMenuItem(i.ToString());
                this.controllerSettings.DropDownItems.Add(item);
            }
            //FansおよびTempsを設定
            CreateFanAndTemps();
            //SidePanelをリフレッシュし、データ表示用ポーリングを開始
            SidePanelRefresh();
        }
        //共通的に利用するクラス
        public  SidePanelFans        Fans  = new SidePanelFans();
        public  SidePanelTemps       Temps = new SidePanelTemps();
        //プライベートなクラス
        private SidePanelConfigDialog SpDialog = new SidePanelConfigDialog();
		private string exeDir;	//このプログラムのexeファイルが格納されているディレクトリ	//V3CSと相違
        #endregion

        #region "SidePanel" //サイドパネルの基本設定
        //変数・定数
        private Double resizeRate = (Double)FanduinaSettings.SidePanelResizeRate;
        private Size   originalSize = new Size();
        private const Int32 MARGIN_TOP    = 24;
        private const Int32 MARGIN_BOTTOM = 10;
        private const Int32 MARGIN_LEFT   = 10;
        private const Int32 MARGIN_RIGHT  = 10;
        private const Int32 MARGIN_MID    = 8;
        private const Int32 CURSOR_LEFT_TOP     = 0;
        private const Int32 CURSOR_RIGHT_TOP    = 1;
        private const Int32 CURSOR_LEFT_BOTTOM  = 2;
        private const Int32 CURSOR_RIGHT_BOTTOM = 3;
		private const Int32 CURSOR_ELSE         = 4;
        //処理関数
        private void  SidePanelResize(Double resizeRate)     //パネルサイズの変更
        {
            Fans.Location = new Point((Int32)(MARGIN_LEFT*resizeRate),
                                          (Int32)(MARGIN_TOP*resizeRate));
            Fans.ResizeRate = resizeRate;
            Fans.RefreshDisp();
            Temps.Location = new Point((Int32)(MARGIN_LEFT*resizeRate),
                                           (Int32)((MARGIN_TOP+MARGIN_MID)*resizeRate)+Fans.Size.Height);
            Temps.ResizeRate = resizeRate;
            Temps.RefreshDisp();
            this.Size = (Size)new Point((Int32)(MARGIN_RIGHT  * resizeRate) + Temps.Location.X + Temps.Width,
                                        (Int32)(MARGIN_BOTTOM * resizeRate) + Temps.Location.Y + Temps.Height);
            sidePanelFrame.Size = this.Size;
            Logo_FanduinaV3.Size = (Size)new Point((Int32)(120 * resizeRate), (Int32)(20 * resizeRate));
            Logo_FanduinaV3.Location = new Point((Int32)(30 * resizeRate), (Int32)(4 * resizeRate));
        }
        private void  FanDispPropSave()                      //ファン表示設定の保存
        {
            for (Int32 i = 0; i < FanduinaSettings.FansFanCount; i++)
            {
                FanduinaSettings.FanDispProp[i].FicNum = Fans.Fan[i].FicNum;
                FanduinaSettings.FanDispProp[i].FanNum = Fans.Fan[i].FanNum;
                FanduinaSettings.FanDispProp[i].FanTextColorInt = Fans.Fan[i].FanTextColor.ToArgb();
                FanduinaSettings.FanDispProp[i].FanBarColorInt = Fans.Fan[i].FanBarColor.ToArgb();
            }
        }
        private void  TempDispPropSave()                     //温度表示設定の保存
        {
            for (Int32 i = 0; i < FanduinaSettings.TempsTempCount; i++)
            {
                FanduinaSettings.TempDispProp[i].FicNum = Temps.Temp[i].FicNum;
                FanduinaSettings.TempDispProp[i].TempNum = Temps.Temp[i].TempNum;
                FanduinaSettings.TempDispProp[i].TempTextColorInt = Temps.Temp[i].TempTextColor.ToArgb();
                FanduinaSettings.TempDispProp[i].TempBarColorInt = Temps.Temp[i].TempBarColor.ToArgb();
            }
        }
        private void  CreateFanAndTemps()                    //ファンパネルと温度パネルを作成
        {
            //Fansを作成
            this.Controls.Add(Fans);
            Fans.Location = new Point(MARGIN_LEFT, MARGIN_TOP);
            Fans.Parent = sidePanelFrame;
            Fans.Create(FanduinaSettings.FansFanCount);
            for (Int32 i = 0; i < FanduinaSettings.FansFanCount; i++)
            {
                Fans.Fan[i].FicNum = FanduinaSettings.FanDispProp[i].FicNum;
                Fans.Fan[i].FanNum = FanduinaSettings.FanDispProp[i].FanNum;
                Fans.Fan[i].FanColor     = Color.FromArgb(FanduinaSettings.FansTitleColorInt);
                Fans.Fan[i].FanTextColor = Color.FromArgb(FanduinaSettings.FanDispProp[i].FanTextColorInt);
                Fans.Fan[i].FanBarColor  = Color.FromArgb(FanduinaSettings.FanDispProp[i].FanBarColorInt);
            }
            //Tempsを作成
            this.Controls.Add(Temps);
            Temps.Location = new Point(MARGIN_LEFT, MARGIN_TOP + Fans.Size.Height + MARGIN_MID);
            Temps.Parent = sidePanelFrame;
            Temps.Create(FanduinaSettings.TempsTempCount);
            for (Int32 i = 0; i < FanduinaSettings.TempsTempCount; i++)
            {
                Temps.Temp[i].FicNum  = FanduinaSettings.TempDispProp[i].FicNum;
                Temps.Temp[i].TempNum = FanduinaSettings.TempDispProp[i].TempNum;
                Temps.Temp[i].TempColor     = Color.FromArgb(FanduinaSettings.TempsTitleColorInt);
                Temps.Temp[i].TempTextColor = Color.FromArgb(FanduinaSettings.TempDispProp[i].TempTextColorInt);
                Temps.Temp[i].TempBarColor  = Color.FromArgb(FanduinaSettings.TempDispProp[i].TempBarColorInt);
            }
            //Fans,Temps作成によりSidePanelのサイズが決定
            SidePanelResize(1.0);
            originalSize = this.Size;//初期サイズを保存
            timerSidePanel.Start();
        }
        private void  DisposeFanAndTemps()                   //ファンパネルと温度パネルを破棄
        {
            timerSidePanel.Stop();
            Fans.Dispose();
            Temps.Dispose();
            Fans = new SidePanelFans();
            Temps = new SidePanelTemps();
        }
        private void  SidePanelRefresh()                     //サイドパネルに設定値を適用
        {
            if (FanduinaSettings.FansFanCount != Fans.FanMax || FanduinaSettings.TempsTempCount != Temps.TempMax)
            {
                DisposeFanAndTemps();
                CreateFanAndTemps();
            }
            Fans.FontName = FanduinaSettings.SidePanelFontName;
            Fans.FontSize = FanduinaSettings.SidePanelFontSize;
            Fans.FanColor = Color.FromArgb(FanduinaSettings.FansTitleColorInt);
            Fans.BackColor = Color.FromArgb(FanduinaSettings.FansBackColorInt);
            Fans.FanColorSameAsTitle = FanduinaSettings.FansSameAsTitleColor;
            Fans.FanColorSameAsTemp = FanduinaSettings.FansSameAsTempColor;
            Temps.FontName = FanduinaSettings.SidePanelFontName;
            Temps.FontSize = FanduinaSettings.SidePanelFontSize;
            Temps.TempColor = Color.FromArgb(FanduinaSettings.TempsTitleColorInt);
            Temps.BackColor = Color.FromArgb(FanduinaSettings.TempsBackColorInt);
            Temps.TempColorSameAsTitle = FanduinaSettings.TempsSameAsTitleColor;
//            this.Opacity = FanduinaSettings.SidePanelOpacity;	//V3CSと相違
			if ( this.TopMost != FanduinaSettings.SidePanelAlwaysOnTop )	//V3CSと相違
			{
	            this.TopMost = FanduinaSettings.SidePanelAlwaysOnTop;
			}
            resizeRate = FanduinaSettings.SidePanelResizeRate;
            SidePanelResize(resizeRate);
        }
        //ポーリングによるファンコン情報の表示（３秒毎）
        private void timerSidePanel_Tick(object sender, EventArgs e)
        {
            DisplayFanconData();
        }
        private void DisplayFanconData()
        {
            try
            {
                for (Int32 i = 0; i < FanduinaSettings.FansFanCount; i++)
                {
                    Int32 ficNum = Fans.Fan[i].FicNum;
                    Int32 fanNum = Fans.Fan[i].FanNum;
                    Fans.Fan[i].FanData = MainForm.Fic[ficNum].FanData[fanNum];
                    if (FanduinaSettings.FansSameAsTempColor)  //ファン表示色を温度と同一に
                    {
                        Int32 tempNum = MainForm.Fic[ficNum].FanParam[fanNum].FanThm - 1;
                        Fans.Fan[i].FanColor = Fans.Fan[i].FanTextColor;
                        Fans.Fan[i].FanColorCommon = true;
                        for (int j = 0; j < Temps.TempMax; j++)
                        {
                            int tempsFicNum = Temps.Temp[j].FicNum;
                            int tempsTempNum = Temps.Temp[j].TempNum;
                            if ((ficNum == tempsFicNum) & (tempNum == tempsTempNum))
                            {
                                Fans.Fan[i].FanColor = Temps.Temp[j].TempTextColor;
                                break;
                            }
                        }
                    }
                }
                for (Int32 i = 0; i < FanduinaSettings.TempsTempCount; i++)
                {
                    Int32 ficNum  = this.Temps.Temp[i].FicNum;
                    Int32 tempNum = this.Temps.Temp[i].TempNum;
                    this.Temps.Temp[i].TempData = MainForm.Fic[ficNum].TempData[tempNum];
                }
            }
            catch { }
        }
        #endregion

        #region "GrowShrink" //拡大縮小・移動等の処理
        //拡大縮小時の化身
        private SidePanelAlias        SpAlias  = new SidePanelAlias();
        private Bitmap spAliasBmp;
        //マウスの現在位置
        private Point mousePoint;
        //拡大縮小時の変数       
        private const Int32 CURSOR_CORNER = 8; //隅とみなすピクセル数
        private Int32 mousePosition;//マウスの位置　0:左上　1:右上　2:左下　3:右下　4:その他
        private Point[] cornerPoint = new Point[4]; //四隅のスクリーン座標
        private Int32[] cornerToLocationX = new Int32[] { -1,  0, -1,  0 };
        private Int32[] cornerToLocationY = new Int32[] { -1, -1,  0,  0 };
        //拡大縮小の処理用ルーチン
        private void  GetCornerPoint()      //四隅のスクリーン座標を取得
        {
            cornerPoint[0] = new Point(this.Left , this.Top   );
            cornerPoint[1] = new Point(this.Right, this.Top   );
            cornerPoint[2] = new Point(this.Left , this.Bottom);
            cornerPoint[3] = new Point(this.Right, this.Bottom);
        }
        private Int32 GetMousePosition()    //マウスがパネルのどこにあるかを取得
        {
            GetCornerPoint();
            for (int corner = CURSOR_LEFT_TOP; corner < CURSOR_ELSE; corner++)
            {
                if ( (Math.Abs(Cursor.Position.X - cornerPoint[corner].X) < CURSOR_CORNER) &&
                     (Math.Abs(Cursor.Position.Y - cornerPoint[corner].Y) < CURSOR_CORNER) )
                {
                    return corner;
                }
            }
            return CURSOR_ELSE;   //隅ではない
        }
        //サイドパネル内でマウスが止まった時：拡大縮小か移動か
        private void SidePanel_MouseHover(object sender, EventArgs e) { Mouse_Hover(sender, e); }
        private void Mouse_Hover(object sender, EventArgs e)
        {
			if ( SpAlias.IsDisposed || SpAlias == null )
			{
	            mousePosition = GetMousePosition();
    	        if (mousePosition == CURSOR_LEFT_TOP | mousePosition == CURSOR_RIGHT_BOTTOM)
        	    {
            	    this.Cursor = Cursors.SizeNWSE;
            	}
           		else if (mousePosition == CURSOR_RIGHT_TOP | mousePosition == CURSOR_LEFT_BOTTOM)
            	{
                	this.Cursor = Cursors.SizeNESW;
            	}
            	else
            	{
                	this.Cursor = Cursors.SizeAll;
            	}
			}
       }
        //マウスのボタンが押されたとき：
        private void SidePanel_MouseDown(object sender, MouseEventArgs e){ Mouse_Down(sender, e); }
        private void Logo_FanduinaV3_MouseDown(object sender, MouseEventArgs e){ Mouse_Down(sender, e); }
        private void Mouse_Down(object sender,MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
				try	{ SpAlias.Dispose(); spAliasBmp.Dispose(); } catch {}
                mousePoint = new Point(e.X, e.Y); 	//マウス位置を記憶する
             	mousePosition = GetMousePosition();	//マウスがコーナーにあるかどうかを判定               
                if ( mousePosition != CURSOR_ELSE )
                {
                    //拡大縮小時はキャプチャした画面を仮に伸縮させる（速度対策）
                    SpAlias = new SidePanelAlias();
                    spAliasBmp = new Bitmap(this.Size.Width, this.Size.Height);
    				Graphics g = Graphics.FromImage(spAliasBmp);
       				g.CopyFromScreen(new Point(this.Left,this.Top), Point.Empty, this.Size); //キャプチャ	//V3CSと相違
 		            g.Dispose();	//V3CSと相違
                    SpAlias.BackgroundImage = spAliasBmp;
//                    SpAlias.TransparencyKey = transColorSpAlias;	//V3CSと相違
					SpAlias.Size = this.Size;
                    SpAlias.Location = this.Location;
					this.TopMost = false;
                    SpAlias.TopMost = true;
                    SpAlias.Show();
//                    this.Opacity = this.Opacity * 0.5;	//V3CSと相違
				}
            }
        }
        //マウスのボタンが離されたとき
        private void SidePanel_MouseUp(object sender, MouseEventArgs e) { Mouse_Up(sender, e); }
        private void Logo_FanduinaV3_MouseUp(object sender, MouseEventArgs e) { Mouse_Up(sender, e); }
        private void Mouse_Up(object sender, MouseEventArgs e)
        {
            if ( mousePosition != CURSOR_ELSE )
            {
                //拡大縮小時は実パネル(SidePanel)を表示
                try
                {
					SidePanelResize(resizeRate);
                	this.Location = new Point( cornerPoint[3 - mousePosition].X + this.Size.Width * cornerToLocationX[mousePosition],
                                           cornerPoint[3 - mousePosition].Y + this.Size.Height * cornerToLocationY[mousePosition] );
                	SpDialog.ResizeRate = resizeRate;
//              	this.Opacity = FanduinaSettings.SidePanelOpacity;	//V3CSと相違
                	FanduinaSettings.SidePanelResizeRate = (Double)resizeRate;
                	FanduinaSettings.SidePanelLocation = this.Location;    //現在位置を記憶
					this.TopMost = FanduinaSettings.SidePanelAlwaysOnTop;	//V3CSと相違
				}
                catch
                {
                }
            }
            else
            {
                //移動時は画面端吸着処理を行う
                const Int32 SNAP = 10;
                if (FanduinaSettings.SidePanelSnapToEdge == true)
                {
                    //画面サイズを取得
                    int Display_Hight = Screen.GetBounds(this).Height;
                    int Display_Width = Screen.GetBounds(this).Width;
                    //画面の端部に吸着
                    if (this.Left <= SNAP & this.Left >= 0)
                    {
                        this.Left = 0;
                    }
                    else if (this.Right >= Display_Width - SNAP & this.Right <= Display_Width)
                    {
                        this.Left = Display_Width - this.Size.Width;
                    }
                    if (this.Top <= SNAP & this.Top >= 0)
                    {
                        this.Top = 0;
                    }
                    else if (this.Bottom >= Display_Hight - SNAP & this.Bottom <= Display_Hight)
                    {
                        this.Top = Display_Hight - this.Size.Height;
                    }
                }
                FanduinaSettings.SidePanelLocation = this.Location;    //現在位置を記憶
            }
			//仮パネル(spAlias)を消去
			try	{ SpAlias.Dispose(); spAliasBmp.Dispose(); } catch {}
        }
        //マウスが動いたとき
        private void SidePanel_MouseMove(object sender, MouseEventArgs e) { Mouse_Move(sender, e); }
        private void Logo_FanduinaV3_MouseMove(object sender, MouseEventArgs e) { Mouse_Move(sender, e); }
        private void Mouse_Move(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                if ( mousePosition != CURSOR_ELSE )
                {
                    if (e.X != mousePoint.X | e.Y != mousePoint.Y)
                    {
                        //拡大縮小時は仮パネル(spAlias)を伸縮
                        double xRate = (Math.Abs((double)(Cursor.Position.X - cornerPoint[3 - mousePosition].X) / (double)this.Size.Width));
                        double yRate = (Math.Abs((double)(Cursor.Position.Y - cornerPoint[3 - mousePosition].Y) / (double)this.Size.Height));
                        if (yRate < xRate) xRate = yRate;
                        resizeRate = xRate * FanduinaSettings.SidePanelResizeRate;
                        if (resizeRate < 0.7)
                        {
                            resizeRate = 0.7;
                            xRate = resizeRate / FanduinaSettings.SidePanelResizeRate;
                        }
                        SpAlias.Size = (Size)new Point((int)((double)this.Size.Width * xRate), (int)((double)this.Size.Height * xRate));
                        SpAlias.Location = new Point(cornerPoint[3 - mousePosition].X + SpAlias.Size.Width * cornerToLocationX[mousePosition],
                                                      cornerPoint[3 - mousePosition].Y + SpAlias.Size.Height * cornerToLocationY[mousePosition]);
                    }
                }
                else
                {
                    //移動時は移動量を加算
                    this.Left += e.X - mousePoint.X;
                    this.Top += e.Y - mousePoint.Y;
                }
            }
            else
            {
                EventArgs ee = null;
                Mouse_Hover(sender,ee);
            }
        }
        //マウスオーバー時非表示の処理
        //マウスオーバーONで、マウスがフォーム内に入ったとき
        private void SidePanel_MouseEnter(object sender, System.EventArgs e)
        {
            if (FanduinaSettings.SidePanelMouseOverInvisible == true)
            {
                //MessageBox.Show("MouseHoverTime=" + timerMouseOver.Interval.ToString() + "msec");
                if ((Control.ModifierKeys & Keys.Control) != Keys.Control)
                {
                    this.Visible = false;
                    timerMouseOver.Stop();
                    timerMouseOver.Enabled = false;
                }
            }
        }
        //マウスオーバーONで、マウスがフォームから出たとき
        private void SidePanel_MouseLeave(object sender, System.EventArgs e)
        {
            if (FanduinaSettings.SidePanelMouseOverInvisible == true)
            {
                timerMouseOver.Interval = (Int32)(FanduinaSettings.SidePanelMouseOverInvisibleTimer * 1000);
                timerMouseOver.Enabled = true;
                timerMouseOver.Start();
            }
        }
        //マウスオーバータイマーイベント
        private void timerMouseOver_Tick(System.Object sender, System.EventArgs e)
        {
            Point mp = new Point(Cursor.Position.X, Cursor.Position.Y);

            //一定時間経過後、マウスが離れていればフォームを再表示
            if (((this.Left - mp.X) > 50 | (mp.X - this.Right) > 50) | ((this.Top - mp.Y) > 50 | (mp.Y - this.Bottom) > 50))
            {
                timerMouseOver.Stop();
                timerMouseOver.Enabled = false;
                this.Visible = true;
                DisplayFanconData();
            }

        }
        #endregion

        #region "ContexMenu" //右クリックからのメニュー等
        public  void sidePanelConfig_Click(object sender, EventArgs e)      //SpDialogの起動
        {
            SpDialog = new SidePanelConfigDialog();
            SpDialog.Show();
            SpDialog.FormClosed += SpDialog_Close;
            SpDialog.ApplyClicked += SpDialog_Apply;
        }
        private void minimizeMenu_Click(object sender, EventArgs e)         //最小化の処理
        {
			this.WindowState = FormWindowState.Minimized;	//V3CSと相違
            this.Visible = false;	//V3CSと相違
        }
        private void readMeFile_Click(object sender, EventArgs e)           //Readmeの表示
        {
            try	//V3CSと相違
            {
    			Process p = new Process();
    			p.StartInfo.FileName = "bash";				
				p.StartInfo.Arguments = FanduinaSettings.ReadmeCommand + 
					                    exeDir + Properties.Resources.ReadmeFileName + "'";
    			p.StartInfo.CreateNoWindow = true;			// コンソール・ウィンドウを開かない
    			p.StartInfo.UseShellExecute = false;		// シェル機能を使用しない
    			p.StartInfo.RedirectStandardOutput = false;	// 標準出力をリダイレクト
				p.Start(); 	// 子プロセスの実行開始								
            }
            catch
            {
                MessageBox.Show(Properties.Resources.ErrWordPadStart, Properties.Resources.ErrorMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        private void exitFanduinaV3_Click(object sender, EventArgs e)       //プログラムの終了
        {
            FanDispPropSave();
            TempDispPropSave();
            this.Close();
        }
        private void controllerSettings_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e) //Ficの表示
        {
            Int32 i = Convert.ToInt32(e.ClickedItem.Text);
            MainForm.Fic[i].Show();
        }
        private void SpDialog_Apply(object sender, EventArgs e)     //SpDailog適用イベントの処理
        {
            SidePanelRefresh();
        }
        private void SpDialog_Close(object sender, EventArgs e)     //SpDialog終了イベントの処理
        {
            //SidePanelRefresh();
            //MessageBox.Show("Memory1 : " + GC.GetTotalMemory(false).ToString());
            SpDialog.Dispose();
            System.GC.Collect();
            //MessageBox.Show("Memory2 : " + GC.GetTotalMemory(false).ToString());
        }
        #endregion
    }
}
 
