﻿using System;
using System.Drawing;
using System.Drawing.Text;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    public partial class SidePanelConfigDialog : Form   //サイドパネル表示設定用のダイアログ・クラス
    {
        //プロパティ
        public Double ResizeRate
        {
            get { return Convert.ToDouble(resizeRate.Value/100); }
            set { resizeRate.Value = Convert.ToDecimal((Int32)(value*100)); }
        }
        //コンストラクタ
        public SidePanelConfigDialog()
        {
            InitializeComponent();
            Load += SidePanelConfigDialog_Load;
        }
        //ロード時初期化
        private void SidePanelConfigDialog_Load(object sender,EventArgs e)
        {
            //fontBoxのプロパティを設定
            fontBoxInit();
            //設定値をロード
            fanCount.Value = Convert.ToDecimal( FanduinaSettings.FansFanCount );
            tempCount.Value = Convert.ToDecimal( FanduinaSettings.TempsTempCount );
            alwaysOnTop.Checked = FanduinaSettings.SidePanelAlwaysOnTop;
            startMinimized.Checked = FanduinaSettings.SidePanelStartMinimize;
            snapToEdge.Checked = FanduinaSettings.SidePanelSnapToEdge;
            mouseOverInvisible.Checked = FanduinaSettings.SidePanelMouseOverInvisible;
            mouseOverInvisibleTimer.Value = Convert.ToDecimal( FanduinaSettings.SidePanelMouseOverInvisibleTimer );
            fontBox.Text = FanduinaSettings.SidePanelFontName.ToString();
            fontSize.Value = Convert.ToDecimal( FanduinaSettings.SidePanelFontSize );
            resizeRate.Value = Convert.ToDecimal((Int32)(FanduinaSettings.SidePanelResizeRate * 100.0));
            opacity.Value = Convert.ToDecimal(100);	//V3CSと相違
            fanColorSameAsTitle.Checked = FanduinaSettings.FansSameAsTitleColor;
            fanColorSameAsTemp.Checked = FanduinaSettings.FansSameAsTempColor;
            fanColorIndividual.Checked = !( FanduinaSettings.FansSameAsTitleColor
                                          | FanduinaSettings.FansSameAsTempColor );
            fanTitleColor.BackColor = Color.FromArgb(FanduinaSettings.FansTitleColorInt);
            fanBackColor.BackColor  = Color.FromArgb(FanduinaSettings.FansBackColorInt);
            tempColorSameAsTitle.Checked = FanduinaSettings.TempsSameAsTitleColor;
            tempColorIndividual.Checked = !FanduinaSettings.TempsSameAsTitleColor;
            tempTitleColor.BackColor = Color.FromArgb(FanduinaSettings.TempsTitleColorInt);
            tempBackColor.BackColor = Color.FromArgb(FanduinaSettings.TempsBackColorInt);
        }

        //fontBox用ルーチン
        private void fontBoxInit()                                                  //fontBoxのプロパティを設定
        {
            fontBox.Items.Clear();
            fontBox.DropDownStyle = ComboBoxStyle.DropDownList;
            fontBox.DrawMode = DrawMode.OwnerDrawFixed;
            fontBox.Items.Add(FanduinaSettings.SidePanelFontName);
            fontBox.Text = FanduinaSettings.SidePanelFontName;
        }
        private void fontBox_Click(System.Object sender, System.EventArgs e)        //フォント名をfontBoxのリストに追加
        {
            if (fontBox.Items.Count <= 1)
            {
                InstalledFontCollection ifc = new InstalledFontCollection();        //InstalledFontCollectionオブジェクトの取得
                FontFamily[] ffs = ifc.Families;                                    //全フォントファミリアを取得
                foreach (FontFamily ff in ffs)
                {
                    if (ff.IsStyleAvailable(FontStyle.Regular))                     //スタイルにRegularが使用できるフォントのみ
                    {
                        fontBox.Items.Add(ff.Name);
                    }
                }
                //MessageBox.Show("Memory : " + GC.GetTotalMemory(false).ToString());
            }
        }
        private void fontBox_DrawItem(object sender, DrawItemEventArgs e)           //fontBoxにフォント名のフォントで描画
        {
            e.DrawBackground();
            ComboBox cmb = (ComboBox)sender;
            string txt = e.Index > -1 ? cmb.Items[e.Index].ToString() : cmb.Text;   //項目に表示する文字列
            Font f = new Font(txt, cmb.Font.Size);                                  //使用するフォント
            Brush b = new SolidBrush(e.ForeColor);                                  //使用するブラシ
            float ym = (e.Bounds.Height - e.Graphics.MeasureString(txt, f).Height) / 2;
            e.Graphics.DrawString(txt, f, b, e.Bounds.X, e.Bounds.Y + ym);          //文字列を描画する

            f.Dispose();
            b.Dispose();

            e.DrawFocusRectangle();                                                 //フォーカスを示す四角形を描画
        }

        //クリックイベント用ルーチン
        private void cancel_Click(object sender, EventArgs e)           //キャンセルボタンクリック時はフォームを閉じる
        {
            this.Close();
        }
        public event EventHandler<EventArgs> ApplyClicked;              //適用ボタンクリックをイベントとして通知させるためのハンドラ
        private void apply_Click(object sender, EventArgs e)            //適用ボタンクリック時は設定を適用し、サイドパネルをリフレッシュ
        {
            FanduinaSettings.FansFanCount   = Convert.ToInt32(fanCount.Value);
            FanduinaSettings.TempsTempCount = Convert.ToInt32(tempCount.Value);
            FanduinaSettings.SidePanelAlwaysOnTop = alwaysOnTop.Checked;
            FanduinaSettings.SidePanelStartMinimize = startMinimized.Checked;
            FanduinaSettings.SidePanelSnapToEdge = snapToEdge.Checked;
            FanduinaSettings.SidePanelMouseOverInvisible = mouseOverInvisible.Checked;
            FanduinaSettings.SidePanelMouseOverInvisibleTimer = Convert.ToDouble(mouseOverInvisibleTimer.Value);
            FanduinaSettings.SidePanelFontName = fontBox.Text.ToString();
            FanduinaSettings.SidePanelFontSize = Convert.ToInt32(fontSize.Value);
            FanduinaSettings.SidePanelResizeRate = Convert.ToDouble(resizeRate.Value) / 100.0;
            FanduinaSettings.SidePanelOpacity = 100.0;	//V3CSと相違
            FanduinaSettings.FansSameAsTitleColor = fanColorSameAsTitle.Checked;
            FanduinaSettings.FansTitleColorInt = fanTitleColor.BackColor.ToArgb();
            FanduinaSettings.FansBackColorInt = fanBackColor.BackColor.ToArgb();
            FanduinaSettings.FansSameAsTempColor = fanColorSameAsTemp.Checked;
            FanduinaSettings.TempsSameAsTitleColor = tempColorSameAsTitle.Checked;
            FanduinaSettings.TempsTitleColorInt = tempTitleColor.BackColor.ToArgb();
            FanduinaSettings.TempsBackColorInt = tempBackColor.BackColor.ToArgb();
            ApplyClicked(this,e);   //イベント通知
            if (mouseOverInvisible.Checked)
            {
                MessageBox.Show(Properties.Resources.MsgMouseOverSet + "\r\n" + Properties.Resources.MsgMouseOverSet2, 
                                Properties.Resources.InformationMessage,
                                MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }
        private void ok_Click(object sender, EventArgs e)               //OKボタンクリック時は設定適用、リフレッシュし、フォームを閉じる
        {
            apply_Click(sender, e);
            this.Close();
        }
        private void fanTitleColor_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            fanTitleColor.BackColor = colorDialog1.Color;
        }
        private void tempTitleColor_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            tempTitleColor.BackColor = colorDialog1.Color;
        }
        private void fanBackColor_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            fanBackColor.BackColor = colorDialog1.Color;
        }
        private void tempBackColor_Click(object sender, EventArgs e)
        {
            colorDialog1.ShowDialog();
            tempBackColor.BackColor = colorDialog1.Color;
        }

    }
}
