﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    // Fan表示用パネルを設定するクラス
    // （メソッド）
    //   Create(fanNum) : fanNum個のファン表示エリアを持った新たなパネルを作る
    //   RefreshDisp()  : サイズ、フォント、色のプロパティに合わせてパネルを再表示する
    // （プロパティ）
    //   FanMax     : 表示するファンの個数
    //   FontName   : フォント名称
    //   FontSize   : フォントサイズ(pt)
    //   ResizeRate : パネルの表示倍率
    //   FanColor   : タイトル文字の色
    //   BackColor  : 背景色
    //   FanColorSameAsTitle : 各Fan表示色をタイトル文字と同一色にする
    //   FanColorSameAsTemp  : 各Fan表示色を温度表示色と同一にする
    //（サブメンバ）
    //   Fan[]      : 個々のファンを表示するクラス
    //   Fan[].RefreshDIsp(): サイズ、フォント、色のプロパティに合わせてファンを再表示する
    //   Fan[].FicNum       : ファンコン番号
    //   Fan[].FanNum       : ファン番号
    //   Fan[].FanName      : ファン名称
    //   Fan[].FanTach      : ファン回転数
    //   Fan[].FanDuty      : ファン出力Duty比
    //   Fan[].ResizeRate   : ファン表示エリアの倍率
    //   Fan[].FontName     : フォント名称
    //   Fan[].FontSize     : フォントサイズ(pt)
    //   Fan[].FanColor     : ファン共通色
    //   Fan[].FanTextColor : 文字色
    //   Fan[].FanBarColor  : グラフ色
    //   Fan[].FanColorCommon : 表示色を共通色にする

    public partial class SidePanelFans : UserControl
    {
        //プロパティ
        public SidePanelFanParts[] Fan = new SidePanelFanParts[24];
        public Int32    FanMax;
        public String   FontName
        {
            get { return fanFontName; }
            set
            {
                fanFontName = value;
            }
        }
        public Int32    FontSize
        {
            get { return fanFontSize; }
            set
            {
                fanFontSize = value;
            }
        }
        public Double   ResizeRate
        {
            get { return rate; }
            set
            {
                rate = value;
            }
        }
        public Color    FanColor
        {
            get { return fanColor; }
            set
            {
                fanColor = value;
                labelFan.ForeColor = value;
                labelRpm.ForeColor = value;
                labelDuty.ForeColor = value;
            }
        }
        public bool     FanColorSameAsTitle
        {
            get { return fanColorSameAsTitle; }
            set
            {
                fanColorSameAsTitle = value;
            }
        }
        public bool     FanColorSameAsTemp
        {
            get { return fanColorSameAsTemp; }
            set
            {
                fanColorSameAsTemp = value;
            }
        }
        //コンストラクタ
        public SidePanelFans()
        {
            InitializeComponent();
            Load += Fans_Load;
        }
        //メソッド
        public void     Create(Int32 fanNum)
        {
            FanMax = fanNum;
            this.Size = originalPanelSize;
            for (int i = 0; i < fanNum; i++)
            {
                Fan[i] = new SidePanelFanParts();
                this.Controls.Add(Fan[i]);
                Fan[i].Location = new Point(0, this.Size.Height);
                Fan[i].ResizeRate = 1.0;
                this.Size = (Size)new Point(this.Size.Width, this.Size.Height + Fan[i].Size.Height);
            }
            this.Size = (Size)new Point(this.Size.Width, this.Size.Height + bottomMargin);
        }
        public void     RefreshDisp()
        {
                this.labelFan.Size = (Size)new Point((Int32)(labelFanSize.Width * rate),
                                                     (Int32)(labelFanSize.Height * rate));
                this.labelFan.Location = new Point((Int32)(labelFanLoc.X * rate),
                                                   (Int32)(labelFanLoc.Y * rate));
                this.labelFan.Font = new Font(labelFanFontName,(float)(labelFanFontSize * rate * fanFontSize / 9.0),FontStyle.Bold);
                this.labelRpm.Size = (Size)new Point((Int32)(labelRpmSize.Width * rate),
                                                     (Int32)(labelRpmSize.Height * rate));
                this.labelRpm.Location = new Point((Int32)(labelRpmLoc.X * rate),
                                                      (Int32)(labelRpmLoc.Y * rate));
                this.labelRpm.Font = new Font(labelRpmFontName,(float)(labelRpmFontSize * rate * fanFontSize / 9.0));
                this.labelDuty.Size = (Size)new Point((Int32)(labelDutySize.Width * rate),
                                                      (Int32)(labelDutySize.Height * rate));
                this.labelDuty.Location = new Point((Int32)(labelDutyLoc.X * rate),
                                                    (Int32)(labelDutyLoc.Y * rate));
                this.labelDuty.Font = new Font(labelDutyFontName,(float)(labelDutyFontSize * rate * fanFontSize / 9.0));

                this.Size = (Size) new Point((Int32)(originalPanelSize.Width*rate),
                                             (Int32)(originalPanelSize.Height*rate));
                for (int i = 0; i < FanMax; i++)
                {
                    Fan[i].FontName = fanFontName;
                    Fan[i].FontSize = fanFontSize;
                    Fan[i].ResizeRate = rate;
                    Fan[i].Location = new Point(0, this.Size.Height);
                    if (fanColorSameAsTitle == true)        //ファン表示色をタイトルと同色に
                    {
                        Fan[i].FanColor = fanColor;
                        Fan[i].FanColorCommon = true;
                    }
                    else if (fanColorSameAsTemp == true)    //ファン表示色を温度計と同色にする処理は、上位のSidePanelで行う
                    {
                        Fan[i].FanColor = fanColor;
                        Fan[i].FanColorCommon = true;
                    }
                    else                                    //ファン表示色は個別設定
                    {
                        Fan[i].FanColorCommon = false;
                    }
                    Fan[i].RefreshDisp();
                    this.Size = (Size)new Point(this.Size.Width,
                                                this.Size.Height + Fan[i].Size.Height);
                }
                this.Size = (Size)new Point(this.Size.Width,
                                            this.Size.Height + (Int32)(bottomMargin * rate));
        }
        //フィールド変数
        private Double  rate;
        private Color   fanColor;
        private bool    fanColorSameAsTitle;
        private bool    fanColorSameAsTemp;
        private String  fanFontName = "tahoma";
        private Int32   fanFontSize = 9;
        //ロード時初期化
        private void Fans_Load(object sender, EventArgs e)
        {
            FanColor = Color.FromArgb(FanduinaSettings.FansTitleColorInt);
            BackColor = Color.FromArgb(FanduinaSettings.FansBackColorInt);
        }
        //ローカル変数
        private Size    originalPanelSize = (Size)new Point(156, 16);
        private Size    labelFanSize = (Size)new Point(50, 16);
        private Point   labelFanLoc = new Point(3, 0);	//V3CSと相違
        private String  labelFanFontName = "tahoma";
        private Int32   labelFanFontSize = 9;
        private Size    labelRpmSize = (Size)new Point(29, 15);
        private Point   labelRpmLoc = new Point(78, 1);	//V3CSと相違
        private String  labelRpmFontName = "tahoma";
        private Int32   labelRpmFontSize = 8;
        private Size    labelDutySize = (Size)new Point(21, 15);
        private Point   labelDutyLoc = new Point(130, 1);	//V3CSと相違
        private String  labelDutyFontName = "tahoma";
        private Int32   labelDutyFontSize = 8;
        private Int32   bottomMargin = 8;
        //クリック時の処理
        private void titleColor_Click(object sender, EventArgs e)
        {
                colorDialogFans.ShowDialog();
                FanduinaSettings.FansTitleColorInt 
                            = colorDialogFans.Color.ToArgb();
                FanColor = colorDialogFans.Color;
        }
        private void backColor_Click(object sender, EventArgs e)
        {
                colorDialogFans.ShowDialog();
                FanduinaSettings.FansBackColorInt
                            = colorDialogFans.Color.ToArgb();
                BackColor = colorDialogFans.Color;
        }
        private void labelFan_Click(object sender, EventArgs e)
        {
            this.titleMenu.Show(this, PointToClient(Cursor.Position));
        }

    }
}


