﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    
    public partial class SidePanelTempParts : UserControl   // 温度計1台分の情報を表示するクラス
    {
        //プロパティ
        public Int32    FicNum          //ファンコン番号（Ficでの登録番号）
        {
            get{return ficNum;}
            set
            {
                ficNum = value;
                foreach (ToolStripMenuItem item in controllerNumMenu.DropDownItems)
                {
                    if (Convert.ToInt32(item.Text) == value)
                    {
                        item.Checked = true;
                    }
                    else
                    {
                        item.Checked = false;
                    }
                }
            }
        }
        public Int32    TempNum         //温度番号
        {
            get{return tempNum;}
            set
            {
                tempNum = value;
                foreach (ToolStripMenuItem item in tempNumMenu.DropDownItems)
                {
                    if (Convert.ToInt32(item.Text) == value+1)
                    {
                        item.Checked = true;
                    }
                    else
                    {
                        item.Checked = false;
                    }
                }
            }
        }
        public TempDataStruct TempData  //温度表示用データ
        {
            set
            {
                dispTempNameText = value.TempName;
                dispTempName.Text = value.TempName;
                dispTempValueVal = value.TempValue;
                dispTempValue.Text = value.TempValue.ToString("0.0");
                TempBarRefresh();
            }
        }
        public String   FontName        //フォント名称
        {
            get { return dispTempFontName; }
            set
            {
                dispTempFontName = value;
            }
        }
        public Int32    FontSize        //フォントサイズ(pt)
        {
            get { return dispTempFontSize; }
            set
            {
                dispTempFontSize = value;
            }
        }
        public Double   ResizeRate      //表示エリアの倍率
        {
            get { return rate; }
            set
            {
                rate = value;
            }
        }
        public Color    TempColor       //温度共通色
        {
            get { return tempColor; }
            set
            {
                tempColor = value;
                if (tempColorCommon == true)
                {
                    dispTempName.ForeColor = value;
                    dispTempValue.ForeColor = value;
                    dispTempBar.BarColor  = value;
                }

            }
        }
        public Color    TempTextColor   //文字個別色
        {
            get { return tempTextColor; }
            set
            {
                tempTextColor = value;
                if (tempColorCommon == false)
                {
                    dispTempName.ForeColor = value;
                    dispTempValue.ForeColor = value;
                }

            }
        }
        public Color    TempBarColor    //グラフ個別色
        {
            get { return tempBarColor; }
            set
            {
                tempBarColor = value;
                if (tempColorCommon == false)
                {
                    dispTempBar.BarColor = value;
                }
            }
        }
        public bool     TempColorCommon //共通色使用の有無
        {
            get { return tempColorCommon; }
            set
            {
                tempColorCommon = value;
                if (value == true)
                {
                    dispTempName.ForeColor = tempColor;
                    dispTempValue.ForeColor = tempColor;
                    dispTempBar.BarColor  = tempColor;
                }
                else
                {
                    dispTempName.ForeColor = tempTextColor;
                    dispTempValue.ForeColor = tempTextColor;
                    dispTempBar.BarColor  = tempBarColor;
                }
            }
        }
        //コンストラクタ
        public SidePanelTempParts()
        {
            InitializeComponent();
            Load += SidePanelTempParts_Load;
        }
        //メソッド
        public void     RefreshDisp()   //サイズ、フォント、色のプロパティに合わせて温度を再表示するメソッド
        {
            this.Size = (Size)new Point((Int32)(dispSize.Width * rate),
                                        (Int32)(dispSize.Height * rate));
            this.dispTempName.Size = (Size)new Point((Int32)(dispTempNameSize.Width * rate),
                                                     (Int32)(dispTempNameSize.Height * rate));
            this.dispTempName.Location = new Point((Int32)(dispTempNameLoc.X * rate),
                                                   (Int32)(dispTempNameLoc.Y * rate));
            this.dispTempName.Font = new Font(dispTempFontName, (float)(dispTempFontSize * rate));
            this.dispTempValue.Size = (Size)new Point((Int32)(dispTempValueSize.Width * rate),
                                                      (Int32)(dispTempValueSize.Height * rate));
            this.dispTempValue.Location = new Point((Int32)(dispTempValueLoc.X * rate),
                                                   (Int32)(dispTempValueLoc.Y * rate));
            this.dispTempValue.Font = new Font(dispTempFontName, (float)(dispTempFontSize * rate));
            this.dispTempBar.ResizeRate = rate;
            this.dispTempBar.Location = new Point((Int32)(dispTempBarLoc.X * rate),
                                                  (Int32)(dispTempBarLoc.Y * rate));
            TempBarRefresh();
        }
        //フィールド変数
        private Int32   ficNum;
        private Int32   tempNum;
        private Color   tempColor;
        private Color   tempTextColor;
        private Color   tempBarColor;
        private bool    tempColorCommon;
        private String  dispTempFontName  = "tahoma";
        private Int32   dispTempFontSize  = 9;
        private String  dispTempNameText = "AAAAAA";
        private Double  dispTempValueVal = 99.9;
        private Double  rate;   //=ResizeRate
        //ロード時初期化
        private void SidePanelTempParts_Load(object sender, EventArgs e)
        {
            this.dispTempBar.Size = dispTempBarSize;
            ToolStripMenuItem item = new ToolStripMenuItem();
            for (Int32 i = 0; i < FanduinaSettings.FicMax; i++)
            {
                item = new ToolStripMenuItem(i.ToString());
                controllerNumMenu.DropDownItems.Add(item);
            }
            for (Int32 i = 0; i < FanduinaSettings.FicTempMax; i++)
            {
                item = new ToolStripMenuItem((i + 1).ToString());
                tempNumMenu.DropDownItems.Add(item);
            }
        }
        //ローカル変数
        private Size    dispSize          = new Size (156, 16);
        private Size    dispTempNameSize  = new Size ( 66, 16);
        private Point   dispTempNameLoc   = new Point(  3,  0);	//V3CSと相違
        private Size    dispTempValueSize = new Size ( 42, 16);
        private Point   dispTempValueLoc  = new Point( 56,  0);	//V3CSと相違
        private Size    dispTempBarSize   = new Size ( 50, 16);
        private Point   dispTempBarLoc    = new Point(100,  0);
        //棒グラフの再描画
        private void    TempBarRefresh()
        {
            if (dispTempValueVal < 20.0)
            {
                dispTempBar.Temp = 0;
            }
            else if (dispTempValueVal > 70.0)
            {
                dispTempBar.Temp = 100;
            }
            else
            {
                dispTempBar.Temp = Convert.ToInt32((dispTempValueVal - 20.0) * 100.0 / (70.0 - 20.0));
            }

        }        
        //クリック時の処理
        private void controllerNumMenu_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            Int32 i = Convert.ToInt32(e.ClickedItem.Text);
            FicNum = i;
        }
        private void tempNumMenu_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            Int32 i = Convert.ToInt32(e.ClickedItem.Text);
            TempNum = i - 1;
        }
        private void colorMenu_Click(object sender, EventArgs e)
        {
            colorDialogTemp.ShowDialog();
            TempTextColor = colorDialogTemp.Color;
            TempBarColor = colorDialogTemp.Color;
        }
        private void dispTempName_MouseHover(object sender, EventArgs e)
        {
            toolTipFicTempNum.SetToolTip(dispTempName, " Cont:" + ficNum.ToString() + " Temp:" + (tempNum + 1).ToString() + " ");
        }
        private void dispTempName_Click(object sender, EventArgs e)
        {
            this.tempDispMenu.Show(this, PointToClient(Cursor.Position));
        }
    }
    
    public class TempBar : UserControl  //温度を棒グラフで表示するクラス
    {
        //プロパティ
        public Int32  Temp               //温度（％）（２０℃が０％、７０℃が１００％）
        {
            get { return temp; }
            set
            {
                temp = value;
                this.BarRefresh();
            }
        }
        public Double ResizeRate         //温度表示エリアの倍率
        {
            get { return resizeRate; }
            set
            {
                resizeRate = value;
                this.BarRefresh();
            }
        }
        public String FontName           //フォント名称
        {
            get { return fontName; }
            set { fontName = value; }
        }
        public Int32  FontSize           //フォントサイズ
        {
            get { return fontSize; }
            set { fontSize = value; }
        }
        public Color  BarColor           //棒グラフの色
        {
            get { return tempBarColor; }
            set { tempBarColor = value; }
        }
        //コンストラクタ
        public TempBar()
        {
            Load += TempBar_Load;
        }
        //ローカル変数
        private Size       originalSize = new Size(50,16);
        private PictureBox tempPict     = new PictureBox();
        private Int32      temp         = 100;
        private Double     resizeRate;
        private String     fontName     = "tahoma";
        private Int32      fontSize     = 9;
        private Color      tempBarColor = Color.Red;
        //処理
        private void TempBar_Load(object sender, EventArgs e) //初期設定
        {
            this.Controls.Add(tempPict);
            tempPict.Location = new Point(0, 0);
            tempPict.Size = originalSize;
            tempPict.BackColor = Color.Transparent;
            BarRefresh();
        }
        private void BarRefresh()   //再表示
        {
            try
            {
                this.Size = new Size((Int32)(originalSize.Width * resizeRate),
                                     (Int32)(originalSize.Height * resizeRate));
                tempPict.Size = this.Size;
                tempPict.Image = new Bitmap(tempPict.Size.Width, tempPict.Size.Height);
                Graphics g = Graphics.FromImage(tempPict.Image);
                Rectangle rectBack = new Rectangle(0, (Int32)(2.5 * resizeRate),
                                         (Int32)((Double)originalSize.Width * resizeRate),
                                         (Int32)((Double)(originalSize.Height - 3.5) * resizeRate));
                SolidBrush sb = new SolidBrush(Color.LightGray);
                g.FillRectangle(sb, rectBack);
                Rectangle rectBar = new Rectangle(0, (Int32)(2.5 * resizeRate),
                                         (Int32)((Double)(temp * originalSize.Width) * resizeRate / 100.0),
                                         (Int32)((Double)(originalSize.Height - 3.5) * resizeRate));
                LinearGradientBrush gb = new LinearGradientBrush(g.VisibleClipBounds,
                                         Color.White, tempBarColor, LinearGradientMode.Vertical);
                g.FillRectangle(gb, rectBar);
                g.Dispose();
                gb.Dispose();
                sb.Dispose();
            }
            catch { }
        }
    }
}