﻿using System;
using System.Drawing;
using System.Windows.Forms;

namespace FanduinaV3CS
{
    // Fan表示用パネルを設定するクラス
    // （メソッド）
    //   Create(tempNum) : tempNum個の温度表示エリアを持った新たなパネルを作る
    //   RefreshDisp()  : サイズ、フォント、色のプロパティに合わせてパネルを再表示する
    // （プロパティ）
    //   TempMax    : 表示する温度の個数
    //   FontName   : フォント名称
    //   FontSize   : フォントサイズ(pt)
    //   ResizeRate : パネルの表示倍率
    //   TempColor  : タイトル文字の色
    //   BackColor  : 背景色
    //   TempColorSameAsTitle : 各Temp表示色をタイトル文字と同一色にする
    //（サブメンバ）
    //   Temp[]      : 個々の温度を表示するクラス
    //   Temp[].RefreshDIsp(): サイズ、フォント、色のプロパティに合わせて温度を再表示する
    //   Temp[].FicNum       : ファンコン番号
    //   Temp[].TempNum      : 温度番号
    //   Temp[].TempName     : 温度名称
    //   Temp[].Temp         : 温度
    //   Temp[].ResizeRate   : 温度表示エリアの倍率
    //   Temp[].FontNmae     : フォント名称
    //   Temp[].FontSize     : フォントサイズ(pt)
    //   Temp[].TempColor    : 温度共通色
    //   Temp[].TempTextColor: 文字色
    //   Temp[].TempBarColor : グラフ色
    //   Temp[].TempColorCommon : 表示色を共通色にする

    public partial class SidePanelTemps : UserControl
    {
        //プロパティ
        public SidePanelTempParts[] Temp = new SidePanelTempParts[16];
        public Int32    TempMax;
        public String   FontName
        {
            get { return tempFontName; }
            set
            {
                tempFontName = value;
            }
        }
        public Int32    FontSize
        {
            get { return tempFontSize; }
            set
            {
                tempFontSize = value;
            }
        }
        public Double   ResizeRate
        {
            get { return rate; }
            set
            {
                rate = value;
            }
        }
        public Color    TempColor
        {
            get { return tempColor; }
            set
            {
                tempColor = value;
                labelTemp.ForeColor = value;
                labelUnit.ForeColor = value;
                labelDeg20.ForeColor = value;
                labelDeg70.ForeColor = value;
            }
        }
        public bool     TempColorSameAsTitle
        {
            get { return tempColorSameAsTitle; }
            set
            {
                tempColorSameAsTitle = value;
            }
        }
        //コンストラクタ
        public SidePanelTemps()
        {
            InitializeComponent();
            Load += Temps_Load;
        }
        //メソッド
        public void     Create(Int32 tempNum)
        {
            TempMax = tempNum;
            this.Size = originalPanelSize;
            for (int i = 0; i < tempNum; i++)
            {
                Temp[i] = new SidePanelTempParts();
                this.Controls.Add(Temp[i]);
                Temp[i].Location = new Point(0, this.Size.Height);
                Temp[i].ResizeRate = 1.0;
                this.Size = (Size)new Point(this.Size.Width, this.Size.Height + Temp[i].Height);
            }
            this.Size = (Size)new Point(this.Size.Width, this.Size.Height + bottomMargin);
        }
        public void     RefreshDisp()
        {
            this.labelTemp.Size = (Size)new Point((Int32)(labelTempSize.Width * rate),
                                                 (Int32)(labelTempSize.Height * rate));
            this.labelTemp.Location = new Point((Int32)(labelTempLoc.X * rate),
                                               (Int32)(labelTempLoc.Y * rate));
            this.labelTemp.Font = new Font(labelTempFontName, (float)(labelTempFontSize * rate * tempFontSize /9.0), FontStyle.Bold);
            this.labelUnit.Size = (Size)new Point((Int32)(labelUnitSize.Width * rate),
                                                 (Int32)(labelUnitSize.Height * rate));
            this.labelUnit.Location = new Point((Int32)(labelUnitLoc.X * rate),
                                                  (Int32)(labelUnitLoc.Y * rate));
            this.labelUnit.Font = new Font(labelUnitFontName, (float)(labelUnitFontSize * rate * tempFontSize /9.0));
            this.labelDeg20.Size = (Size)new Point((Int32)(labelDeg20Size.Width * rate),
                                                  (Int32)(labelDeg20Size.Height * rate));
            this.labelDeg20.Location = new Point((Int32)(labelDeg20Loc.X * rate),
                                                  (Int32)(labelDeg20Loc.Y * rate));
            this.labelDeg20.Font = new Font(labelDeg20FontName, (float)(labelDeg20FontSize * rate * tempFontSize /9.0));
            this.labelDeg70.Size = (Size)new Point((Int32)(labelDeg70Size.Width * rate),
                                                  (Int32)(labelDeg70Size.Height * rate));
            this.labelDeg70.Location = new Point((Int32)(labelDeg70Loc.X * rate),
                                                  (Int32)(labelDeg70Loc.Y * rate));
            this.labelDeg70.Font = new Font(labelDeg70FontName, (float)(labelDeg70FontSize * rate * tempFontSize /9.0));

            this.Size = (Size)new Point((Int32)(originalPanelSize.Width * rate),
                                         (Int32)(originalPanelSize.Height * rate));
            for (int i = 0; i < TempMax; i++)
            {
                Temp[i].FontName = tempFontName;
                Temp[i].FontSize = tempFontSize;
                Temp[i].ResizeRate = rate;
                Temp[i].Location = new Point(0, this.Size.Height);
                if (tempColorSameAsTitle == true)
                {
                    Temp[i].TempColor = tempColor;
                    Temp[i].TempColorCommon = true;
                }
                else
                {
                    Temp[i].TempColorCommon = false;
                }
                Temp[i].RefreshDisp();
                this.Size = (Size)new Point(this.Size.Width,
                                            this.Size.Height + Temp[i].Size.Height);
            }
            this.Size = (Size)new Point(this.Size.Width,
                                        this.Size.Height + (Int32)(bottomMargin * rate));
        }
        //フィールド変数
        private Double  rate;
        private Color   tempColor;
        private bool    tempColorSameAsTitle;
        private String  tempFontName = "tahoma";
        private Int32   tempFontSize = 9;
        //ロード時初期化
        private void Temps_Load(object sender, EventArgs e)
        {
            TempColor = Color.FromArgb(FanduinaSettings.TempsTitleColorInt);
            BackColor = Color.FromArgb(FanduinaSettings.TempsBackColorInt);
        }
        //ローカル変数
        private Size    originalPanelSize = (Size)new Point(156, 16);
        private Size    labelTempSize = (Size)new Point(50, 16);
        private Point   labelTempLoc = new Point(3, 0);	//V3CSと相違
        private String  labelTempFontName = "tahoma";
        private Int32   labelTempFontSize = 9;
        private Size    labelUnitSize = (Size)new Point(29, 15);
        private Point   labelUnitLoc = new Point(80, 2);	//V3CSと相違
        private String  labelUnitFontName = "tahoma";
        private Int32   labelUnitFontSize = 8;
        private Size    labelDeg20Size = (Size)new Point(24, 15);
        private Point   labelDeg20Loc = new Point(92, 1);	//V3CSと相違
        private String  labelDeg20FontName = "tahoma";
        private Int32   labelDeg20FontSize = 8;
        private Size    labelDeg70Size = (Size)new Point(24, 15);
        private Point   labelDeg70Loc = new Point(132, 1);
        private String  labelDeg70FontName = "tahoma";
        private Int32   labelDeg70FontSize = 8;
        private Int32   bottomMargin = 8;
        //クリック時の処理
        private void titleColor_Click(object sender, EventArgs e)
        {
            colorDialogTemps.ShowDialog();
            FanduinaSettings.TempsTitleColorInt
                        = colorDialogTemps.Color.ToArgb();
            TempColor = colorDialogTemps.Color;
        }
        private void backColor_Click(object sender, EventArgs e)
        {
            colorDialogTemps.ShowDialog();
            FanduinaSettings.TempsBackColorInt
                        = colorDialogTemps.Color.ToArgb();
            BackColor = colorDialogTemps.Color;
        }
        private void labelTemp_Click(object sender, EventArgs e)
        {
            this.titleMenu.Show(this, PointToClient(Cursor.Position));
        }
    }
}
